#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Brian Scholer <@briantist>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_psmodule_info
short_description: Gather information about PowerShell Modules
description:
  - Gather information about PowerShell Modules including information from PowerShellGet.
options:
  name:
    description:
      - The name of the module to retrieve.
      - Supports any wildcard pattern supported by C(Get-Module).
      - If omitted then all modules will returned.
    type: str
    default: '*'
  repository:
    description:
      - The name of the PSRepository the modules were installed from.
      - This acts as a filter against the modules that would be returned based on the I(name) option.
      - Modules that were not installed from a repository will not be returned if this option is set.
      - Only modules installed from a registered repository will be returned.
      - If the repository was re-registered after module installation with a new C(SourceLocation), this will not match.
    type: str
requirements:
  - C(PowerShellGet) module
seealso:
  - module: community.windows.win_psrepository_info
  - module: community.windows.win_psscript_info
author:
  - Brian Scholer (@briantist)
'''

EXAMPLES = r'''
- name: Get info about all modules on the system
  community.windows.win_psmodule_info:

- name: Get info about the ScheduledTasks module
  community.windows.win_psmodule_info:
    name: ScheduledTasks

- name: Get info about networking modules
  community.windows.win_psmodule_info:
    name: Net*

- name: Get info about all modules installed from the PSGallery repository
  community.windows.win_psmodule_info:
    repository: PSGallery
  register: gallery_modules

- name: Update all modules retrieved from above example
  community.windows.win_psmodule:
    name: "{{ item }}"
    state: latest
  loop: "{{ gallery_modules.modules | map(attribute=name) }}"

- name: Get info about all modules on the system
  community.windows.win_psmodule_info:
  register: all_modules

- name: Find modules installed from a repository that isn't registered now
  set_fact:
    missing_repository_modules: "{{
      all_modules
      | json_query('modules[?repository!=null && repository==repository_source_location].{name: name, version: version, repository: repository}')
      | list
    }}"

- debug:
    var: missing_repository_modules
'''

RETURN = r'''
modules:
  description:
    - A list of modules (or an empty list is there are none).
  returned: always
  type: list
  elements: dict
  contains:
    name:
      description:
        - The name of the module.
      type: str
      sample: PSReadLine
    version:
      description:
        - The module version.
      type: str
      sample: 1.2.3
    guid:
      description:
        - The GUID of the module.
      type: str
      sample: 74c9fd30-734b-4c89-a8ae-7727ad21d1d5
    path:
      description:
        - The path to the module.
      type: str
      sample: 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\PKI\PKI.psd1'
    module_base:
      description:
        - The path that contains the module's files.
      type: str
      sample: 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\PKI'
    installed_location:
      description:
        - The path where the module is installed.
        - This should have the same value as C(module_base) but only has a value when the module was installed via PowerShellGet.
      type: str
      sample: 'C:\Program Files\WindowsPowerShell\Modules\posh-git\0.7.1'
    exported_aliases:
      description:
        - The aliases exported from the module.
      type: list
      elements: str
      sample:
        - glu
        - slu
    exported_cmdlets:
      description:
        - The cmdlets exported from the module.
      type: list
      elements: str
      sample:
        - Get-Certificate
        - Get-PfxData
    exported_commands:
      description:
        - All of the commands exported from the module. Includes functions, cmdlets, and aliases.
      type: list
      elements: str
      sample:
        - glu
        - Get-LocalUser
    exported_dsc_resources:
      description:
        - The DSC resources exported from the module.
      type: list
      elements: str
      sample:
        - xWebAppPool
        - xWebSite
    exported_format_files:
      description:
        - The format files exported from the module.
      type: list
      elements: path
      sample:
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsCmdlets.Format.ps1xml'
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsConfig.Format.ps1xml'
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsClientPSProvider.Format.ps1xml'
    exported_functions:
      description:
        - The functions exported from the module.
      type: list
      elements: str
      sample:
        - New-VirtualDisk
        - New-Volume
    exported_type_files:
      description:
        - The type files exported from the module.
      type: list
      elements: path
      sample:
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsCmdlets.Types.ps1xml'
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsConfig.Types.ps1xml'
        - 'C:\WINDOWS\system32\WindowsPowerShell\v1.0\Modules\DnsClient\DnsClientPSProvider.Types.ps1xml'
    exported_variables:
      description:
        - The variables exported from the module.
      type: list
      elements: str
      sample:
        - GitPromptScriptBlock
    exported_workflows:
      description:
        - The workflows exported from the module.
        - This will not be present if targeting a PowerShell 7 interpreter.
      type: list
      elements: str
    access_mode:
      description:
        - The module's access mode. See U(https://docs.microsoft.com/en-us/dotnet/api/system.management.automation.moduleaccessmode)
      type: str
      sample: ReadWrite
    module_type:
      description:
        - The module's type. See U(https://docs.microsoft.com/en-us/dotnet/api/system.management.automation.moduletype)
      type: str
      sample: Script
    procoessor_architecture:
      description:
        - The module's processor architecture. See U(https://docs.microsoft.com/en-us/dotnet/api/system.reflection.processorarchitecture)
      type: str
      sample: Amd64
    author:
      description:
        - The author of the module.
      type: str
      sample: Warren Frame
    copyright:
      description:
        - The copyright of the module.
      type: str
      sample: '(c) 2016 Warren F. All rights reserved.'
    company_name:
      description:
        - The company name of the module.
      type: str
      sample: Microsoft Corporation
    description:
      description:
        - The description of the module.
      type: str
      sample: Provides cmdlets to work with local users and local groups
    clr_version:
      description:
        - The CLR version of the module.
      type: str
      sample: '4.0'
    compatible_ps_editions:
      description:
        - The PS Editions the module is compatible with.
      type: list
      elements: str
      sample:
        - Desktop
    dependencies:
      description:
        - The modules required by this module.
      type: list
      elements: str
    dot_net_framework_version:
      description:
        - The .Net Framework version of the module.
      type: str
      sample: '4.6.1'
    file_list:
      description:
        - The files included in the module.
      type: list
      elements: path
      sample:
        - 'C:\Program Files\WindowsPowerShell\Modules\PowerShellGet\1.6.0\PSModule.psm1'
        - 'C:\Program Files\WindowsPowerShell\Modules\PowerShellGet\1.6.0\PSGet.Format.ps1xml'
        - 'C:\Program Files\WindowsPowerShell\Modules\PowerShellGet\1.6.0\PSGet.Resource.psd1'
    help_info_uri:
      description:
        - The help info address of the module.
      type: str
      sample: 'https://go.microsoft.com/fwlink/?linkid=390823'
    icon_uri:
      description:
        - The address of the icon of the module.
      type: str
      sample: 'https://raw.githubusercontent.com/powershell/psscriptanalyzer/master/logo.png'
    license_uri:
      description:
        - The address of the license for the module.
      type: str
      sample: 'https://github.com/PowerShell/xPendingReboot/blob/master/LICENSE'
    project_uri:
      description:
        - The address of the module's project.
      type: str
      sample: 'https://github.com/psake/psake'
    repository_source_location:
      description:
        - The source location of the repository where the module was installed from.
      type: str
      sample: 'https://www.powershellgallery.com/api/v2'
    repository:
      description:
        - The PSRepository where the module was installed from.
        - This value is not historical. It depends on the PSRepositories that are registered now for the current user.
        - The C(repository_source_location) must match the current source location of a registered repository to get a repository name.
        - If there is no match, then this value will match C(repository_source_location).
      type: str
      sample: PSGallery
    release_notes:
      description:
        - The module's release notes. This is a free text field and no specific format should be assumed.
      type: str
      sample: |
        ## 1.4.6
        - Update `HelpInfoUri` to point to the latest content

        ## 1.4.5
        - Bug fix for deadlock when getting parameters in an event

        ## 1.4.4
        - Bug fix when installing modules from private feeds
    installed_date:
      description:
        - The date the module was installed.
      type: str
      sample: '2018-02-14T17:55:34.9620740-05:00'
    published_date:
      description:
        - The date the module was published.
      type: str
      sample: '2017-03-15T04:18:09.0000000'
    updated_date:
      description:
        - The date the module was last updated.
      type: str
      sample: '2019-12-31T09:20:02.0000000'
    log_pipeline_execution_details:
      description:
        - Determines whether pipeline execution detail events should be logged.
      type: bool
    module_list:
      description:
        - A list of modules packaged with this module.
        - This value is not often returned and the modules are not automatically processed.
      type: list
      elements: dict
      contains:
        name:
          description:
            - The name of the module.
            - This may also be a path to the module file.
          type: str
          sample: '.\WindowsUpdateLog.psm1'
        guid:
          description:
            - The GUID of the module.
          type: str
          sample: 82fdb72c-ecc5-4dfd-b9d5-83cf6eb9067f
        version:
          description:
            - The minimum version of the module.
          type: str
          sample: '2.0'
        maximum_version:
          description:
            - The maximum version of the module.
          type: str
          sample: '2.9'
        required_version:
          description:
            - The exact version of the module required.
          type: str
          sample: '3.1.4'
    nested_modules:
      description:
        - A list of modules nested with and loaded into the scope of this module.
        - This list contains full module objects, so each item can have all of the properties listed here, including C(nested_modules).
      type: list
      elements: dict
    required_modules:
      description:
        - A list of modules required by this module.
        - This list contains full module objects, so each item can have all of the properties listed here, including C(required_modules).
        - These module objects may not contain full information however, so you may see different results than if you had directly queried the module.
      type: list
      elements: dict
    required_assemblies:
      description:
        - A list of assemblies that the module requires.
        - The values may be a simple name or a full path.
      type: str
      sample:
        - Microsoft.Management.Infrastructure.CimCmdlets.dll
        - Microsoft.Management.Infrastructure.Dll
    package_management_provider:
      description:
        - If the module was installed from PowerShellGet, this is the package management provider used.
      type: str
      sample: NuGet
    power_shell_host_name:
      description:
        - The name of the PowerShell host that the module requires.
      type: str
      sample: Windows PowerShell ISE Host
    power_shell_host_version:
      description:
        - The version of the PowerShell host that the module requires.
      type: str
      sample: '1.1'
    power_shell_version:
      description:
        - The minimum version of PowerShell that the module requires.
      type: str
      sample: '5.1'
    prefix:
      description:
        - The default prefix applied to C(Verb-Noun) commands exported from the module, resulting in C(Verb-PrefixNoun) names.
      type: str
    private_data:
      description:
        - Arbitrary private data used by the module. This is typically defined in the module manifest.
        - This module limits the depth of the data returned for module types other than C(Script) and C(Manifest).
        - The C(PSData) is commonly supplied and provides metadata for PowerShellGet but those fields are surfaced in top-level properties as well.
      type: dict
      sample:
        PSData:
          LicenseUri: https://example.com/module/LICENSE
          ProjectUri: https://example.com/module/
          ReleaseNotes: |
            v2 - Fixed some bugs
            v1 - First release
          Tags:
            - networking
            - serialization
    root_module:
      description:
        - The root module as defined in the manifest.
        - This may be a module name, filename, or full path.
      type: str
      sample: WindowsErrorReporting.psm1
    scripts:
      description:
        - A list of scripts (C(.ps1) files) that run in the caller's session state when the module is imported.
        - This value comes from the C(ScriptsToProcess) field in the module's manifest.
      type: list
      sample:
        - PrepareEnvironment.ps1
        - InitializeData.ps1
    tags:
      description:
        - The tags defined in the module's C(PSData) metadata.
      type: list
      elements: str
      sample:
        - networking
        - serialization
        - git
        - dsc
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_controller_migrations_info
short_description: Information module for organizations _devices _controller _migrations
description:
  - Get all organizations _devices _controller _migrations.
  - Retrieve device migration statuses in an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  serials:
    description:
      - Serials query parameter. A list of Meraki Serials for which to retrieve migrations.
    elements: str
    type: list
  networkIds:
    description:
      - NetworkIds query parameter. Filter device migrations by network IDs.
    elements: str
    type: list
  target:
    description:
      - Target query parameter. Filter device migrations by target destination.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesControllerMigrations
    description: Complete reference of the getOrganizationDevicesControllerMigrations
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-controller-migrations
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_controller_migrations,
  - Paths used are
    get /organizations/{organizationId}/devices/controller/migrations,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _controller _migrations
  cisco.meraki.organizations_devices_controller_migrations_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serials: []
    networkIds: []
    target: string
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "target": "string",
          "createdAt": "string",
          "migratedAt": "string"
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_sensor_commands
short_description: Resource module for devices _sensor _commands
description:
  - Manage operation create of the resource devices _sensor _commands.
  - Sends a command to a sensor.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  operation:
    description: Operation to run on the sensor. 'enableDownstreamPower', 'disableDownstreamPower',
      and 'cycleDownstreamPower' turn power on/off to the device that is connected
      downstream of an MT40 power monitor. 'refreshData' causes an MT15 or MT40 device
      to upload its latest readings so that they are immediately available in the
      Dashboard API.
    type: str
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sensor createDeviceSensorCommand
    description: Complete reference of the createDeviceSensorCommand API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-sensor-command
notes:
  - SDK Method used are
    sensor.Sensor.create_device_sensor_command,
  - Paths used are
    post /devices/{serial}/sensor/commands,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_sensor_commands:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    operation: disableDownstreamPower
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "commandId": "string",
      "createdAt": "string",
      "completedAt": "string",
      "createdBy": {
        "adminId": "string",
        "name": "string",
        "email": "string"
      },
      "operation": "string",
      "status": "string",
      "errors": [
        "string"
      ]
    }
"""



#!/usr/bin/python

# Copyright (c) 2018, Martin Migasiewicz <migasiew.nk@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: launchd
author:
  - Martin Migasiewicz (@martinm82)
short_description: Manage macOS services
version_added: 1.0.0
description:
  - Manage launchd services on target macOS hosts.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the service.
    type: str
    required: true
  plist:
    description:
      - Name of the V(.plist) file for the service.
      - Defaults to V({name}.plist).
    type: str
    version_added: 10.1.0
  state:
    description:
      - V(started)/V(stopped) are idempotent actions that do not run commands unless necessary.
      - C(launchd) does not support V(restarted) nor V(reloaded) natively. These states trigger a stop/start (restarted) or
        an unload/load (reloaded).
      - V(restarted) unloads and loads the service before start to ensure that the latest job definition (plist) is used.
      - V(reloaded) unloads and loads the service to ensure that the latest job definition (plist) is used. Whether a service
        is started or stopped depends on the content of the definition file.
    type: str
    choices: [reloaded, restarted, started, stopped, unloaded]
  enabled:
    description:
      - Whether the service should start on boot.
      - B(At least one of state and enabled are required).
    type: bool
  force_stop:
    description:
      - Whether the service should not be restarted automatically by launchd.
      - Services might have the C(KeepAlive) attribute set to V(true) in a launchd configuration. In case this is set to V(true),
        stopping a service causes that C(launchd) starts the service again.
      - Set this option to V(true) to let this module change the C(KeepAlive) attribute to V(false).
    type: bool
    default: false
notes:
  - A user must privileged to manage services using this module.
requirements:
  - A system managed by launchd
  - The plistlib Python library
"""

EXAMPLES = r"""
- name: Make sure spotify webhelper is started
  community.general.launchd:
    name: com.spotify.webhelper
    state: started

- name: Deploy custom memcached job definition
  template:
    src: org.memcached.plist.j2
    dest: /Library/LaunchDaemons/org.memcached.plist

- name: Run memcached
  community.general.launchd:
    name: org.memcached
    state: started

- name: Stop memcached
  community.general.launchd:
    name: org.memcached
    state: stopped

- name: Stop memcached
  community.general.launchd:
    name: org.memcached
    state: stopped
    force_stop: true

- name: Restart memcached
  community.general.launchd:
    name: org.memcached
    state: restarted

- name: Unload memcached
  community.general.launchd:
    name: org.memcached
    state: unloaded

- name: restart sshd
  community.general.launchd:
    name: com.openssh.sshd
    plist: ssh.plist
    state: restarted
"""

RETURN = r"""
status:
  description: Metadata about service status.
  returned: always
  type: dict
  sample:
    {
      "current_pid": "-",
      "current_state": "stopped",
      "previous_pid": "82636",
      "previous_state": "running"
    }
"""

import os
import plistlib
from abc import ABCMeta, abstractmethod
from time import sleep

from ansible.module_utils.basic import AnsibleModule


class ServiceState:
    UNKNOWN = 0
    LOADED = 1
    STOPPED = 2
    STARTED = 3
    UNLOADED = 4

    @staticmethod
    def to_string(state):
        strings = {
            ServiceState.UNKNOWN: "unknown",
            ServiceState.LOADED: "loaded",
            ServiceState.STOPPED: "stopped",
            ServiceState.STARTED: "started",
            ServiceState.UNLOADED: "unloaded",
        }
        return strings[state]


class Plist:
    def __init__(self, module, service, filename=None):
        self.__changed = False
        self.__service = service
        if filename is not None:
            self.__filename = filename
        else:
            self.__filename = f"{service}.plist"

        state, pid, dummy, dummy = LaunchCtlList(module, self.__service).run()

        self.__file = self.__find_service_plist(self.__filename)
        if self.__file is None:
            msg = f"Unable to find the plist file {self.__filename} for service {self.__service}"
            if pid is None and state == ServiceState.UNLOADED:
                msg += " and it was not found among active services"
            module.fail_json(msg=msg)
        self.__update(module)

    @staticmethod
    def __find_service_plist(filename):
        """Finds the plist file associated with a service"""

        launchd_paths = [
            os.path.join(os.getenv("HOME"), "Library/LaunchAgents"),
            "/Library/LaunchAgents",
            "/Library/LaunchDaemons",
            "/System/Library/LaunchAgents",
            "/System/Library/LaunchDaemons",
        ]

        for path in launchd_paths:
            try:
                files = os.listdir(path)
            except OSError:
                continue

            if filename in files:
                return os.path.join(path, filename)
        return None

    def __update(self, module):
        self.__handle_param_enabled(module)
        self.__handle_param_force_stop(module)

    def __read_plist_file(self, module):
        service_plist = {}
        try:
            with open(self.__file, "rb") as plist_fp:
                service_plist = plistlib.load(plist_fp)
        except Exception as e:
            module.fail_json(msg=f"Failed to read plist file {self.__file} due to {e}")
        return service_plist

    def __write_plist_file(self, module, service_plist=None):
        if not service_plist:
            service_plist = {}

        try:
            with open(self.__file, "wb") as plist_fp:
                plistlib.dump(service_plist, plist_fp)
        except Exception as e:
            module.fail_json(msg=f"Failed to write to plist file  {self.__file} due to {e}")

    def __handle_param_enabled(self, module):
        if module.params["enabled"] is not None:
            service_plist = self.__read_plist_file(module)

            # Enable/disable service startup at boot if requested
            # Launchctl does not expose functionality to set the RunAtLoad
            # attribute of a job definition. So we parse and modify the job
            # definition plist file directly for this purpose.
            if module.params["enabled"] is not None:
                enabled = service_plist.get("RunAtLoad", False)
                if module.params["enabled"] != enabled:
                    service_plist["RunAtLoad"] = module.params["enabled"]

                    # Update the plist with one of the changes done.
                    if not module.check_mode:
                        self.__write_plist_file(module, service_plist)
                        self.__changed = True

    def __handle_param_force_stop(self, module):
        if module.params["force_stop"] is not None:
            service_plist = self.__read_plist_file(module)

            # Set KeepAlive to false in case force_stop is defined to avoid
            # that the service gets restarted when stopping was requested.
            if module.params["force_stop"] is not None:
                keep_alive = service_plist.get("KeepAlive", False)
                if module.params["force_stop"] and keep_alive:
                    service_plist["KeepAlive"] = not module.params["force_stop"]

                    # Update the plist with one of the changes done.
                    if not module.check_mode:
                        self.__write_plist_file(module, service_plist)
                        self.__changed = True

    def is_changed(self):
        return self.__changed

    def get_file(self):
        return self.__file


class LaunchCtlTask(metaclass=ABCMeta):
    WAITING_TIME = 5  # seconds

    def __init__(self, module, service, plist):
        self._module = module
        self._service = service
        self._plist = plist
        self._launch = self._module.get_bin_path("launchctl", True)

    def run(self):
        """Runs a launchd command like 'load', 'unload', 'start', 'stop', etc.
        and returns the new state and pid.
        """
        self.runCommand()
        return self.get_state()

    @abstractmethod
    def runCommand(self):
        pass

    def get_state(self):
        rc, out, err = self._launchctl("list")
        if rc != 0:
            self._module.fail_json(msg=f"Failed to get status of {self._launch}")

        state = ServiceState.UNLOADED
        service_pid = "-"
        status_code = None
        for line in out.splitlines():
            if line.strip():
                pid, last_exit_code, label = line.split("\t")
                if label.strip() == self._service:
                    service_pid = pid
                    status_code = last_exit_code

                    # From launchctl man page:
                    # If the number [...] is negative, it represents  the
                    # negative of the signal which killed the job.  Thus,
                    # "-15" would indicate that the job was terminated with
                    # SIGTERM.
                    if last_exit_code not in ["0", "-2", "-3", "-9", "-15"]:
                        # Something strange happened and we have no clue in
                        # which state the service is now. Therefore we mark
                        # the service state as UNKNOWN.
                        state = ServiceState.UNKNOWN
                    elif pid != "-":
                        # PID seems to be an integer so we assume the service
                        # is started.
                        state = ServiceState.STARTED
                    else:
                        # Exit code is 0 and PID is not available so we assume
                        # the service is stopped.
                        state = ServiceState.STOPPED
                    break
        return (state, service_pid, status_code, err)

    def start(self):
        rc, out, err = self._launchctl("start")
        # Unfortunately launchd does not wait until the process really started.
        sleep(self.WAITING_TIME)
        return (rc, out, err)

    def stop(self):
        rc, out, err = self._launchctl("stop")
        # Unfortunately launchd does not wait until the process really stopped.
        sleep(self.WAITING_TIME)
        return (rc, out, err)

    def restart(self):
        # TODO: check for rc, out, err
        self.stop()
        return self.start()

    def reload(self):
        # TODO: check for rc, out, err
        self.unload()
        return self.load()

    def load(self):
        return self._launchctl("load")

    def unload(self):
        return self._launchctl("unload")

    def _launchctl(self, command):
        service_or_plist = (
            self._plist.get_file()
            if command in ["load", "unload"]
            else self._service
            if command in ["start", "stop"]
            else ""
        )

        rc, out, err = self._module.run_command(f"{self._launch} {command} {service_or_plist}")

        if rc != 0:
            msg = f"Unable to {command} '{self._service}' ({self._plist.get_file()}): '{err}'"
            self._module.fail_json(msg=msg)

        return (rc, out, err)


class LaunchCtlStart(LaunchCtlTask):
    def __init__(self, module, service, plist):
        super().__init__(module, service, plist)

    def runCommand(self):
        state, dummy, dummy, dummy = self.get_state()

        if state in (ServiceState.STOPPED, ServiceState.LOADED):
            self.reload()
            self.start()
        elif state == ServiceState.STARTED:
            # In case the service is already in started state but the
            # job definition was changed we need to unload/load the
            # service and start the service again.
            if self._plist.is_changed():
                self.reload()
                self.start()
        elif state == ServiceState.UNLOADED:
            self.load()
            self.start()
        elif state == ServiceState.UNKNOWN:
            # We are in an unknown state, let's try to reload the config
            # and start the service again.
            self.reload()
            self.start()


class LaunchCtlStop(LaunchCtlTask):
    def __init__(self, module, service, plist):
        super().__init__(module, service, plist)

    def runCommand(self):
        state, dummy, dummy, dummy = self.get_state()

        if state == ServiceState.STOPPED:
            # In case the service is stopped and we might later decide
            # to start it, we need to reload the job definition by
            # forcing an unload and load first.
            # Afterwards we need to stop it as it might have been
            # started again (KeepAlive or RunAtLoad).
            if self._plist.is_changed():
                self.reload()
                self.stop()
        elif state in (ServiceState.STARTED, ServiceState.LOADED):
            if self._plist.is_changed():
                self.reload()
            self.stop()
        elif state == ServiceState.UNKNOWN:
            # We are in an unknown state, let's try to reload the config
            # and stop the service gracefully.
            self.reload()
            self.stop()


class LaunchCtlReload(LaunchCtlTask):
    def __init__(self, module, service, plist):
        super().__init__(module, service, plist)

    def runCommand(self):
        state, dummy, dummy, dummy = self.get_state()

        if state == ServiceState.UNLOADED:
            # launchd throws an error if we do an unload on an already
            # unloaded service.
            self.load()
        else:
            self.reload()


class LaunchCtlUnload(LaunchCtlTask):
    def __init__(self, module, service, plist):
        super().__init__(module, service, plist)

    def runCommand(self):
        state, dummy, dummy, dummy = self.get_state()
        self.unload()


class LaunchCtlRestart(LaunchCtlReload):
    def __init__(self, module, service, plist):
        super().__init__(module, service, plist)

    def runCommand(self):
        super().runCommand()
        self.start()


class LaunchCtlList(LaunchCtlTask):
    def __init__(self, module, service):
        super().__init__(module, service, None)

    def runCommand(self):
        # Do nothing, the list functionality is done by the
        # base class run method.
        pass


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            plist=dict(type="str"),
            state=dict(type="str", choices=["reloaded", "restarted", "started", "stopped", "unloaded"]),
            enabled=dict(type="bool"),
            force_stop=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
        required_one_of=[
            ["state", "enabled"],
        ],
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    service = module.params["name"]
    plist_filename = module.params["plist"]
    action = module.params["state"]
    err = ""
    result = {
        "name": service,
        "changed": False,
        "status": {},
    }

    # We will tailor the plist file in case one of the options
    # (enabled, force_stop) was specified.
    plist = Plist(module, service, plist_filename)
    result["changed"] = plist.is_changed()

    # Gather information about the service to be controlled.
    state, pid, dummy, dummy = LaunchCtlList(module, service).run()
    result["status"]["previous_state"] = ServiceState.to_string(state)
    result["status"]["previous_pid"] = pid

    # Map the actions to specific tasks
    tasks = {
        "started": LaunchCtlStart(module, service, plist),
        "stopped": LaunchCtlStop(module, service, plist),
        "restarted": LaunchCtlRestart(module, service, plist),
        "reloaded": LaunchCtlReload(module, service, plist),
        "unloaded": LaunchCtlUnload(module, service, plist),
    }

    status_code = "0"
    # Run the requested task
    if not module.check_mode:
        state, pid, status_code, err = tasks[action].run()

    result["status"]["current_state"] = ServiceState.to_string(state)
    result["status"]["current_pid"] = pid
    result["status"]["status_code"] = status_code
    result["status"]["error"] = err

    # restarted and reloaded always perform commands unconditionally, so they always change state
    if (
        action in ("restarted", "reloaded")
        or result["status"]["current_state"] != result["status"]["previous_state"]
        or result["status"]["current_pid"] != result["status"]["previous_pid"]
    ):
        result["changed"] = True
    elif module.check_mode and result["status"]["current_state"] != action:
        result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_camera_quality_and_retention
short_description: Resource module for devices _camera _quality _and _retention
description:
  - Manage operation update of the resource devices _camera _quality _and _retention.
  - Update quality and retention settings for the given camera.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  audioRecordingEnabled:
    description: Boolean indicating if audio recording is enabled(true) or disabled(false)
      on the camera.
    type: bool
  motionBasedRetentionEnabled:
    description: Boolean indicating if motion-based retention is enabled(true) or
      disabled(false) on the camera.
    type: bool
  motionDetectorVersion:
    description: The version of the motion detector that will be used by the camera.
      Only applies to Gen 2 cameras. Defaults to v2.
    type: int
  profileId:
    description: The ID of a quality and retention profile to assign to the camera.
      The profile's settings will override all of the per-camera quality and retention
      settings. If the value of this parameter is null, any existing profile will
      be unassigned from the camera.
    type: str
  quality:
    description: Quality of the camera. Can be one of 'Standard', 'High', 'Enhanced'
      or 'Ultra'. Not all qualities are supported by every camera model.
    type: str
  resolution:
    description: Resolution of the camera. Can be one of '1280x720', '1920x1080',
      '1080x1080', '2112x2112', '2880x2880', '2688x1512' or '3840x2160'.Not all resolutions
      are supported by every camera model.
    type: str
  restrictedBandwidthModeEnabled:
    description: Boolean indicating if restricted bandwidth is enabled(true) or disabled(false)
      on the camera. This setting does not apply to MV2 cameras.
    type: bool
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for camera updateDeviceCameraQualityAndRetention
    description: Complete reference of the updateDeviceCameraQualityAndRetention API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-camera-quality-and-retention
notes:
  - SDK Method used are
    camera.Camera.update_device_camera_quality_and_retention,
  - Paths used are
    put /devices/{serial}/camera/qualityAndRetention,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_camera_quality_and_retention:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    audioRecordingEnabled: false
    motionBasedRetentionEnabled: false
    motionDetectorVersion: 2
    profileId: '1234'
    quality: Standard
    resolution: 1280x720
    restrictedBandwidthModeEnabled: false
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "profileId": {},
      "motionBasedRetentionEnabled": true,
      "audioRecordingEnabled": true,
      "restrictedBandwidthModeEnabled": true,
      "quality": "string",
      "resolution": "string",
      "motionDetectorVersion": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_switch_ports_overview_info
short_description: Information module for organizations _switch _ports _overview
description:
  - Get all organizations _switch _ports _overview. - > Returns the counts of all
    active ports for the requested timespan, grouped by speed. An active port is a
    port that at any point during the timeframe is observed to be connected to a responsive
    device and isn't configured to be disabled. For a port that is observed at multiple
    speeds during the timeframe, it will be counted at the highest speed observed.
    The number of inactive ports, and the total number of ports are also provided.
    Only ports on switches online during the timeframe will be represented and a port
    is only guaranteed to be present if its switch was online for at least 6 hours
    of the timeframe.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 186 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be greater than or equal to 12 hours and be less than or
        equal to 186 days. The default is 1 day.
    type: float
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch getOrganizationSwitchPortsOverview
    description: Complete reference of the getOrganizationSwitchPortsOverview API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-switch-ports-overview
notes:
  - SDK Method used are
    switch.Switch.get_organization_switch_ports_overview,
  - Paths used are
    get /organizations/{organizationId}/switch/ports/overview,
"""

EXAMPLES = r"""
- name: Get all organizations _switch _ports _overview
  cisco.meraki.organizations_switch_ports_overview_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    t0: string
    t1: string
    timespan: 0
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "total": 0,
      "byStatus": {
        "active": {
          "total": 0,
          "byMediaAndLinkSpeed": {
            "rj45": {
              "total": 0,
              "10": 0,
              "100": 0,
              "1000": 0,
              "2500": 0,
              "5000": 0,
              "10000": 0
            },
            "sfp": {
              "total": 0,
              "100": 0,
              "1000": 0,
              "10000": 0,
              "20000": 0,
              "25000": 0,
              "40000": 0,
              "50000": 0,
              "100000": 0
            }
          }
        },
        "inactive": {
          "total": 0,
          "byMedia": {
            "rj45": {
              "total": 0
            },
            "sfp": {
              "total": 0
            }
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Brian Scholer <@briantist>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_pssession_configuration
short_description: Manage PSSession Configurations
description:
  - Register, unregister, and modify PSSession Configurations for PowerShell remoting.
options:
  name:
    description:
      - The name of the session configuration to manage.
    type: str
    required: yes
  state:
    description:
      - The desired state of the configuration.
    type: str
    choices:
      - present
      - absent
    default: present
  guid:
    description:
      - The GUID (UUID) of the session configuration file.
      - This value is metadata, so it only matters if you use it externally.
      - If not set, a value will be generated automatically.
      - Acceptable GUID formats are flexible. Any string of 32 hexadecimal digits will be accepted, with all hyphens C(-) and opening/closing C({}) ignored.
      - See also I(lenient_config_fields).
    type: raw
  schema_version:
    description:
      - The schema version of the session configuration file.
      - If not set, a value will be generated automatically.
      - Must be a valid .Net System.Version string.
    type: raw
  author:
    description:
      - The author of the session configuration.
      - This value is metadata and does not affect the functionality of the session configuration.
      - If not set, a value may be generated automatically.
      - See also I(lenient_config_fields).
    type: str
  description:
    description:
      - The description of the session configuration.
      - This value is metadata and does not affect the functionality of the session configuration.
      - See also I(lenient_config_fields).
    type: str
  company_name:
    description:
      - The company that authored the session configuration.
      - This value is metadata and does not affect the functionality of the session configuration.
      - If not set, a value may be generated automatically.
      - See also I(lenient_config_fields).
    type: str
  copyright:
    description:
      - The copyright statement of the session configuration.
      - This value is metadata and does not affect the functionality of the session configuration.
      - If not set, a value may be generated automatically.
      - See also I(lenient_config_fields).
    type: str
  session_type:
    description:
      - Controls what type of session this is.
    type: str
    choices:
      - default
      - empty
      - restricted_remote_server
  transcript_directory:
    description:
      - Automatic session transcripts will be written to this directory.
    type: path
  run_as_virtual_account:
    description:
      - If C(yes) the session runs as a virtual account.
      - Do not use I(run_as_credential_username) and I(run_as_credential_password) to specify a virtual account.
    type: bool
  run_as_virtual_account_groups:
    description:
      - If I(run_as_virtual_account=yes) this is a list of groups to add the virtual account to.
    type: list
    elements: str
  mount_user_drive:
      description:
        - If C(yes) the session creates and mounts a user-specific PSDrive for use with file transfers.
      type: bool
  user_drive_maximum_size:
    description:
      - The maximum size of the user drive in bytes.
      - Must fit into an Int64.
    type: raw
  group_managed_service_account:
    description:
      - If the session will run as a group managed service account (gMSA) then this is the name.
      - Do not use I(run_as_credential_username) and I(run_as_credential_password) to specify a gMSA.
    type: str
  scripts_to_process:
    description:
      - A list of paths to script files ending in C(.ps1) that should be applied to the session.
    type: list
    elements: str
  role_definitions:
    description:
      - A dict defining the roles for JEA sessions.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/session-configurations#role-definitions).
    type: dict
  required_groups:
    description:
      - For JEA sessions, defines conditional access rules about which groups a connecting user must belong to.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/session-configurations#conditional-access-rules).
    type: dict
  language_mode:
    description:
      - Determines the language mode of the PowerShell session.
    type: str
    choices:
      - no_language
      - restricted_language
      - constrained_language
      - full_language
  execution_policy:
    description:
      - The execution policy controlling script execution in the PowerShell session.
    type: str
    choices:
      - default
      - remote_signed
      - restricted
      - undefined
      - unrestricted
  powershell_version:
    description:
      - The minimum required PowerShell version for this session.
      - Must be a valid .Net System.Version string.
      - This only works if targeting PowerShell 5.1, PowerShell 7.x does not support this option.
    type: raw
  modules_to_import:
    description:
      - A list of modules that should be imported into the session.
      - Any valid PowerShell module spec can be used here, so simple str names or dicts can be used.
      - If a dict is used, no snake_case conversion is done, so the original PowerShell names must be used.
    type: list
    elements: raw
  visible_aliases:
    description:
      - The aliases that can be used in the session.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/role-capabilities).
    type: list
    elements: str
  visible_cmdlets:
    description:
      - The cmdlets that can be used in the session.
      - The elements can be simple names or complex command specifications.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/role-capabilities).
    type: list
    elements: raw
  visible_functions:
    description:
      - The functions that can be used in the session.
      - The elements can be simple names or complex command specifications.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/role-capabilities).
    type: list
    elements: raw
  visible_external_commands:
    description:
      - The external commands and scripts that can be used in the session.
      - For more information see U(https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/role-capabilities).
    type: list
    elements: str
  alias_definitions:
    description:
      - A dict that defines aliases for each session.
    type: dict
  function_definitions:
    description:
      - A dict that defines functions for each session.
    type: dict
  variable_definitions:
    description:
      - A list of dicts where each elements defines a variable for each session.
    type: list
    elements: dict
  environment_variables:
    description:
      - A dict that defines environment variables for each session.
    type: dict
  types_to_process:
    description:
      - Paths to type definition files to process for each session.
    type: list
    elements: path
  formats_to_process:
    description:
      - Paths to format definition files to process for each session.
    type: list
    elements: path
  assemblies_to_load:
    description:
      - The assemblies that should be loaded into each session.
    type: list
    elements: str
  processor_architecure:
    description:
      - The processor architecture of the session (32 bit vs. 64 bit).
    type: str
    choices:
      - amd64
      - x86
  access_mode:
    description:
      - Controls whether the session configuration allows connection from the C(local) machine only, both local and C(remote), or none (C(disabled)).
    type: str
    choices:
      - disabled
      - local
      - remote
  use_shared_process:
    description:
      - If C(yes) then the session shares a process for each session.
    type: bool
  thread_apartment_state:
    description:
      - The apartment state for the PowerShell session.
    type: str
    choices:
      - mta
      - sta
  thread_options:
    description:
      - Sets thread options for the session.
    type: str
    choices:
      - default
      - reuse_thread
      - use_current_thread
      - use_new_thread
  startup_script:
    description:
      - A script that gets run on session startup.
    type: path
  maximum_received_data_size_per_command_mb:
    description:
      - Sets the maximum received data size per command in MB.
      - Must fit into a double precision floating point value.
    type: raw
  maximum_received_object_size_mb:
    description:
      - Sets the maximum object size in MB.
      - Must fit into a double precision floating point value.
    type: raw
  security_descriptor_sddl:
    description:
      - An SDDL string that controls which users and groups can connect to the session.
      - If I(role_definitions) is specified the security descriptor will be set based on that.
      - If this option is not specified the default security descriptor will be applied.
    type: str
  run_as_credential_username:
    description:
      - Used to set a RunAs account for the session. All commands executed in the session will be run as this user.
      - To use a gMSA, see I(group_managed_service_account).
      - To use a virtual account, see I(run_as_virtual_account) and I(run_as_virtual_account_groups).
      - Status will always be C(changed) when a RunAs credential is set because the password cannot be retrieved for comparison.
    type: str
  run_as_credential_password:
    description:
      - The password for I(run_as_credential_username).
    type: str
  lenient_config_fields:
    description:
      - Some fields used in the session configuration do not affect its function, and are sometimes auto-generated when not specified.
      - To avoid unnecessarily changing the configuration on each run, the values of these options will only be enforced when they are explicitly specified.
    type: list
    elements: str
    default:
      - guid
      - author
      - company_name
      - copyright
      - description
  async_timeout:
    description:
      - Sets a timeout for how long in seconds to wait for asynchronous module execution and waiting for the connection to recover.
      - Replicates the functionality of the C(async) keyword.
      - Has no effect in check mode.
    type: int
    default: 300
  async_poll:
    description:
      - Sets a delay in seconds between each check of the asynchronous execution status.
      - Replicates the functionality of the C(poll) keyword.
      - Has no effect in check mode.
      - I(async_poll=0) is not supported.
    type: int
    default: 1
notes:
  - This module will restart the WinRM service on any change. This will terminate all WinRM connections including those by other Ansible runs.
  - Internally this module uses C(async) when not in check mode to ensure things go smoothly when restarting the WinRM service.
  - The standard C(async) and C(poll) keywords cannot be used; instead use the I(async_timeout) and I(async_poll) options to control asynchronous execution.
  - Options that don't list a default value here will use the defaults of C(New-PSSessionConfigurationFile) and C(Register-PSSessionConfiguration).
  - If a value can be specified in both a session config file and directly in the session options, this module will prefer the setting be in the config file.
seealso:
  - name: C(New-PSSessionConfigurationFile) Reference
    description: Details and defaults for options that end up in the session configuration file.
    link: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/new-pssessionconfigurationfile
  - name: C(Register-PSSessionConfiguration) Reference
    description: Details and defaults for options that are not specified in the session config file.
    link: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/register-pssessionconfiguration
  - name: PowerShell Just Enough Administration (JEA)
    description: Refer to the JEA documentation for advanced usage of some options
    link: https://docs.microsoft.com/en-us/powershell/scripting/learn/remoting/jea/overview
  - name: About Session Configurations
    description: General information about session configurations.
    link: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_session_configurations
  - name: About Session Configuration Files
    description: General information about session configuration files.
    link: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_session_configuration_files
author:
  - Brian Scholer (@briantist)
'''

EXAMPLES = r'''
- name: Register a session configuration that loads modules automatically
  community.windows.win_pssession_configuration:
    name: WebAdmin
    modules_to_import:
      - WebAdministration
      - IISAdministration
    description: This endpoint has IIS modules pre-loaded

- name: Set up an admin endpoint with a restricted execution policy
  community.windows.win_pssession_configuration:
    name: GloboCorp.Admin
    company_name: Globo Corp
    description: Admin Endpoint
    execution_policy: restricted

- name: Create a complex JEA endpoint
  community.windows.win_pssession_configuration:
    name: RBAC.Endpoint
    session_type: restricted_remote_server
    run_as_virtual_account: true
    transcript_directory: '\\server\share\Transcripts'
    language_mode: no_language
    execution_policy: restricted
    role_definitions:
      'CORP\IT Support':
        RoleCapabilities:
          - PasswordResetter
          - EmployeeOffboarder
      'CORP\Webhosts':
        RoleCapabilities: IISAdmin
    visible_functions:
      - tabexpansion2
      - help
    visible_cmdlets:
      - Get-Help
      - Name: Get-Service
        Parameters:
          - Name: DependentServices
          - Name: RequiredServices
          - Name: Name
            ValidateSet:
              - WinRM
              - W3SVC
              - WAS
    visible_aliases:
      - gsv
    state: present

- name: Remove a session configuration
  community.windows.win_pssession_configuration:
    name: UnusedEndpoint
    state: absent

- name: Set a sessions configuration with tweaked async values
  community.windows.win_pssession_configuration:
    name: MySession
    description: A sample session
    async_timeout: 500
    async_poll: 5
'''

RETURN = r'''
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_server_profile_template
short_description: Manage Server Profile Templates in Cisco Intersight
description:
  - Create, update, and delete Server Profile Templates on Cisco Intersight.
  - Server Profile Templates define a reusable configuration that can be used to derive multiple Server Profiles.
  - Policies are attached to the template via the PolicyBucket sub-resource endpoint.
  - To derive Server Profiles from a template, use the derive_profiles playbook included in this collection.
    See U(https://github.com/CiscoDevNet/intersight-ansible/blob/master/playbooks/derive_profiles.yml) for details.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/server/ProfileTemplate/post/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Templates and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Server Profile Template.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description of the Server Profile Template.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  target_platform:
    description:
      - The platform for which the server profile template is applicable.
      - C(standalone) for servers operating in Standalone mode.
      - C(fi-attached) for servers attached to a Fabric Interconnect managed by Intersight.
      - C(unified-edge) for Unified Edge servers.
    type: str
    choices: [standalone, fi-attached, unified-edge]
    default: standalone
  adapter_policy:
    description:
      - Name of Adapter Configuration Policy to associate with this template.
    type: str
  bios_policy:
    description:
      - Name of BIOS Policy to associate with this template.
    type: str
  boot_order_policy:
    description:
      - Name of Boot Order Policy to associate with this template.
    type: str
  certificate_policy:
    description:
      - Name of Certificate Management Policy to associate with this template.
    type: str
  device_connector_policy:
    description:
      - Name of Device Connector Policy to associate with this template.
    type: str
  drive_security_policy:
    description:
      - Name of Drive Security Policy to associate with this template.
    type: str
  firmware_policy:
    description:
      - Name of Firmware Policy to associate with this template.
    type: str
  ipmi_over_lan_policy:
    description:
      - Name of IPMI over LAN Policy to associate with this template.
    type: str
  lan_connectivity_policy:
    description:
      - Name of LAN Connectivity Policy to associate with this template.
    type: str
  ldap_policy:
    description:
      - Name of LDAP Policy to associate with this template.
    type: str
  local_user_policy:
    description:
      - Name of Local User Policy to associate with this template.
    type: str
  memory_policy:
    description:
      - Name of Memory Policy to associate with this template.
    type: str
  network_connectivity_policy:
    description:
      - Name of Network Connectivity Policy to associate with this template.
    type: str
  ntp_policy:
    description:
      - Name of NTP Policy to associate with this template.
    type: str
  persistent_memory_policy:
    description:
      - Name of Persistent Memory Policy to associate with this template.
    type: str
  power_policy:
    description:
      - Name of Power Policy to associate with this template.
    type: str
  san_connectivity_policy:
    description:
      - Name of SAN Connectivity Policy to associate with this template.
    type: str
  sd_card_policy:
    description:
      - Name of SD Card Policy to associate with this template.
    type: str
  serial_over_lan_policy:
    description:
      - Name of Serial over LAN Policy to associate with this template.
    type: str
  smtp_policy:
    description:
      - Name of SMTP Policy to associate with this template.
    type: str
  snmp_policy:
    description:
      - Name of SNMP Policy to associate with this template.
    type: str
  ssh_policy:
    description:
      - Name of SSH Policy to associate with this template.
    type: str
  storage_policy:
    description:
      - Name of Storage Policy to associate with this template.
    type: str
  syslog_policy:
    description:
      - Name of Syslog Policy to associate with this template.
    type: str
  thermal_policy:
    description:
      - Name of Thermal Policy to associate with this template.
    type: str
  virtual_kvm_policy:
    description:
      - Name of Virtual KVM Policy to associate with this template.
    type: str
  virtual_media_policy:
    description:
      - Name of Virtual Media Policy to associate with this template.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic Server Profile Template
  cisco.intersight.intersight_server_profile_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "SPT-Standalone-Basic"
    description: "Basic standalone server profile template"
    target_platform: standalone
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Create a Server Profile Template with policies
  cisco.intersight.intersight_server_profile_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "SPT-FI-Production"
    description: "FI-Attached template with policies"
    target_platform: fi-attached
    bios_policy: "BIOS-Production"
    boot_order_policy: "Boot-UEFI"
    firmware_policy: "FW-Latest"
    ntp_policy: "NTP-Corp"
    power_policy: "Power-Grid-Redundancy"
    storage_policy: "Storage-M2-Boot"
    lan_connectivity_policy: "LAN-Prod"
    san_connectivity_policy: "SAN-Prod"
    state: present

- name: Delete a Server Profile Template
  cisco.intersight.intersight_server_profile_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SPT-Standalone-Basic"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "SPT-FI-Production",
        "ObjectType": "server.ProfileTemplate",
        "TargetPlatform": "FIAttached",
        "PolicyBucket": [
            {
                "Moid": "6937ec186275723101f63708",
                "ObjectType": "bios.Policy"
            },
            {
                "Moid": "69a42f0770686131014f259c",
                "ObjectType": "firmware.Policy"
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import (
    IntersightModule,
    intersight_argument_spec,
    resolve_policy_bucket,
    sync_policy_bucket,
)


POLICY_MAPPING = {
    'adapter_policy': {'resource_path': '/adapter/ConfigPolicies', 'object_type': 'adapter.ConfigPolicy'},
    'bios_policy': {'resource_path': '/bios/Policies', 'object_type': 'bios.Policy'},
    'boot_order_policy': {'resource_path': '/boot/PrecisionPolicies', 'object_type': 'boot.PrecisionPolicy'},
    'certificate_policy': {'resource_path': '/certificatemanagement/Policies', 'object_type': 'certificatemanagement.Policy'},
    'device_connector_policy': {'resource_path': '/deviceconnector/Policies', 'object_type': 'deviceconnector.Policy'},
    'drive_security_policy': {'resource_path': '/storage/DriveSecurityPolicies', 'object_type': 'storage.DriveSecurityPolicy'},
    'firmware_policy': {'resource_path': '/firmware/Policies', 'object_type': 'firmware.Policy'},
    'ipmi_over_lan_policy': {'resource_path': '/ipmioverlan/Policies', 'object_type': 'ipmioverlan.Policy'},
    'lan_connectivity_policy': {'resource_path': '/vnic/LanConnectivityPolicies', 'object_type': 'vnic.LanConnectivityPolicy'},
    'ldap_policy': {'resource_path': '/iam/LdapPolicies', 'object_type': 'iam.LdapPolicy'},
    'local_user_policy': {'resource_path': '/iam/EndPointUserPolicies', 'object_type': 'iam.EndPointUserPolicy'},
    'memory_policy': {'resource_path': '/memory/Policies', 'object_type': 'memory.Policy'},
    'network_connectivity_policy': {'resource_path': '/networkconfig/Policies', 'object_type': 'networkconfig.Policy'},
    'ntp_policy': {'resource_path': '/ntp/Policies', 'object_type': 'ntp.Policy'},
    'persistent_memory_policy': {'resource_path': '/memory/PersistentMemoryPolicies', 'object_type': 'memory.PersistentMemoryPolicy'},
    'power_policy': {'resource_path': '/power/Policies', 'object_type': 'power.Policy'},
    'san_connectivity_policy': {'resource_path': '/vnic/SanConnectivityPolicies', 'object_type': 'vnic.SanConnectivityPolicy'},
    'sd_card_policy': {'resource_path': '/sdcard/Policies', 'object_type': 'sdcard.Policy'},
    'serial_over_lan_policy': {'resource_path': '/sol/Policies', 'object_type': 'sol.Policy'},
    'smtp_policy': {'resource_path': '/smtp/Policies', 'object_type': 'smtp.Policy'},
    'snmp_policy': {'resource_path': '/snmp/Policies', 'object_type': 'snmp.Policy'},
    'ssh_policy': {'resource_path': '/ssh/Policies', 'object_type': 'ssh.Policy'},
    'storage_policy': {'resource_path': '/storage/StoragePolicies', 'object_type': 'storage.StoragePolicy'},
    'syslog_policy': {'resource_path': '/syslog/Policies', 'object_type': 'syslog.Policy'},
    'thermal_policy': {'resource_path': '/thermal/Policies', 'object_type': 'thermal.Policy'},
    'virtual_kvm_policy': {'resource_path': '/kvm/Policies', 'object_type': 'kvm.Policy'},
    'virtual_media_policy': {'resource_path': '/vmedia/Policies', 'object_type': 'vmedia.Policy'},
}

TARGET_PLATFORM_MAP = {
    'standalone': 'Standalone',
    'fi-attached': 'FIAttached',
    'unified-edge': 'UnifiedEdgeServer',
}


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        target_platform=dict(type='str', choices=['standalone', 'fi-attached', 'unified-edge'], default='standalone'),
        adapter_policy=dict(type='str'),
        bios_policy=dict(type='str'),
        boot_order_policy=dict(type='str'),
        certificate_policy=dict(type='str'),
        device_connector_policy=dict(type='str'),
        drive_security_policy=dict(type='str'),
        firmware_policy=dict(type='str'),
        ipmi_over_lan_policy=dict(type='str'),
        lan_connectivity_policy=dict(type='str'),
        ldap_policy=dict(type='str'),
        local_user_policy=dict(type='str'),
        memory_policy=dict(type='str'),
        network_connectivity_policy=dict(type='str'),
        ntp_policy=dict(type='str'),
        persistent_memory_policy=dict(type='str'),
        power_policy=dict(type='str'),
        san_connectivity_policy=dict(type='str'),
        sd_card_policy=dict(type='str'),
        serial_over_lan_policy=dict(type='str'),
        smtp_policy=dict(type='str'),
        snmp_policy=dict(type='str'),
        ssh_policy=dict(type='str'),
        storage_policy=dict(type='str'),
        syslog_policy=dict(type='str'),
        thermal_policy=dict(type='str'),
        virtual_kvm_policy=dict(type='str'),
        virtual_media_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Create/update the template itself
    resource_path = '/server/ProfileTemplates'

    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()
        api_target_platform = TARGET_PLATFORM_MAP[intersight.module.params['target_platform']]
        intersight.api_body['TargetPlatform'] = api_target_platform

    intersight.configure_policy_or_profile(resource_path=resource_path)
    template_response = intersight.result['api_response']
    template_moid = template_response.get('Moid') if template_response else None

    # Sync PolicyBucket via the sub-resource endpoint
    if template_moid and intersight.module.params['state'] == 'present':
        organization_name = intersight.module.params['organization']
        desired_bucket = resolve_policy_bucket(intersight, organization_name, POLICY_MAPPING)
        current_bucket = template_response.get('PolicyBucket') or []
        bucket_path = f'/server/ProfileTemplates/{template_moid}/PolicyBucket'
        sync_policy_bucket(intersight, bucket_path, desired_bucket, current_bucket)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_drive_security_policy
short_description: Drive Security Policy configuration for Cisco Intersight
description:
  - Manages Drive Security Policy configuration on Cisco Intersight.
  - A policy to configure drive security settings for Cisco Intersight managed servers.
  - Supports both Manual key management and Remote key management (KMIP).
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Drive Security Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Drive Security Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  manual_key:
    description:
      - Configuration for manual key management.
      - Use this for local key management with manual passphrase.
      - Either C(manual_key) or C(remote_key) must be specified, but not both.
    type: dict
    suboptions:
      new_key:
        description:
          - New Security Key Passphrase to be configured on the server.
          - The passphrase must be at least 8 characters long and include at least one uppercase letter, one lowercase letter, one number, and one
            special character.
        type: str
        required: true
      existing_key:
        description:
          - Current Security Key Passphrase which is already configured on the server.
          - Required only if drive security is already enabled with manual key.
          - The passphrase must be at least 8 characters long and include at least one uppercase letter, one lowercase letter, one number, and one
            special character.
        type: str
  remote_key:
    description:
      - Configuration for remote key management using KMIP server.
      - Use this for remote key management with KMIP protocol.
      - Either C(manual_key) or C(remote_key) must be specified, but not both.
    type: dict
    suboptions:
      primary_server:
        description:
          - Primary KMIP server configuration.
          - At least one of C(primary_server) or C(secondary_server) must be enabled.
        type: dict
        suboptions:
          enable_drive_security:
            description:
              - Enables/disables the primary KMIP server.
            type: bool
            required: true
          ip_address:
            description:
              - The IP address or hostname of the KMIP server.
              - Can be an IPv4 address, IPv6 address, or hostname.
              - Hostnames are valid only when Inband is configured for the CIMC address.
              - Required when C(enable_drive_security) is C(true).
            type: str
          port:
            description:
              - The port to which the KMIP client should connect.
              - Valid range is 1024-65535.
            type: int
            default: 5696
          timeout:
            description:
              - The timeout before which the KMIP client should connect.
              - Valid range is 1-250 seconds.
            type: int
            default: 60
      secondary_server:
        description:
          - Secondary KMIP server configuration.
          - At least one of C(primary_server) or C(secondary_server) must be enabled.
        type: dict
        suboptions:
          enable_drive_security:
            description:
              - Enables/disables the secondary KMIP server.
            type: bool
            required: true
          ip_address:
            description:
              - The IP address or hostname of the KMIP server.
              - Can be an IPv4 address, IPv6 address, or hostname.
              - Hostnames are valid only when Inband is configured for the CIMC address.
              - Required when C(enable_drive_security) is C(true).
            type: str
          port:
            description:
              - The port to which the KMIP client should connect.
              - Valid range is 1024-65535.
            type: int
            default: 5696
          timeout:
            description:
              - The timeout before which the KMIP client should connect.
              - Valid range is 1-250 seconds.
            type: int
            default: 60
      server_certificate:
        description:
          - Server Public Root CA Certificate in base64 encoded format.
          - Required when using remote key management (KMIP).
        type: str
        required: true
      use_authentication:
        description:
          - Enables/disables authentication for communicating with KMIP server.
          - When enabled, authentication is mandatory.
        type: bool
        default: false
      username:
        description:
          - The username for the KMIP server login.
          - Required when C(use_authentication) is C(true).
        type: str
      password:
        description:
          - The password for the KMIP server login.
          - Optional parameter for KMIP authentication.
        type: str
      existing_key:
        description:
          - Current Security Key Passphrase which is already configured on the server.
          - Required only if drive security is already enabled with manual key and switching to KMIP.
          - The passphrase must be at least 8 characters long and include at least one uppercase letter, one lowercase letter, one number, and one
            special character.
        type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Drive Security Policy with Manual Key
  cisco.intersight.intersight_drive_security_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "manual-drive-security-policy"
    description: "Drive security policy with manual key management"
    manual_key:
      new_key: "MyS3cur3P@ssw0rd"
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Create Drive Security Policy with Manual Key (updating existing)
  cisco.intersight.intersight_drive_security_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "manual-drive-security-policy"
    description: "Drive security policy with manual key management"
    manual_key:
      new_key: "MyN3wS3cur3P@ssw0rd"
      existing_key: "MyS3cur3P@ssw0rd"
    state: present

- name: Create Drive Security Policy with KMIP (Primary Server Only)
  cisco.intersight.intersight_drive_security_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "kmip-drive-security-policy"
    description: "Drive security policy with KMIP"
    remote_key:
      primary_server:
        enable_drive_security: true
        ip_address: "192.168.1.100"
        port: 5696
        timeout: 60
      secondary_server:
        enable_drive_security: false
      server_certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
      use_authentication: false
    state: present

- name: Create Drive Security Policy with KMIP (Both Servers with Authentication)
  cisco.intersight.intersight_drive_security_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "kmip-auth-drive-security-policy"
    description: "Drive security policy with KMIP and authentication"
    remote_key:
      primary_server:
        enable_drive_security: true
        ip_address: "192.168.1.100"
        port: 5696
        timeout: 60
      secondary_server:
        enable_drive_security: true
        ip_address: "192.168.1.101"
        port: 5696
        timeout: 60
      server_certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
      use_authentication: true
      username: "kmip_user"
      password: "kmip_password"
    state: present

- name: Delete Drive Security Policy
  cisco.intersight.intersight_drive_security_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "manual-drive-security-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "manual-drive-security-policy",
        "ObjectType": "storage.DriveSecurityPolicy",
        "Description": "Drive security policy with manual key management",
        "KeySetting": {
            "ClassId": "storage.KeySetting",
            "ObjectType": "storage.KeySetting",
            "KeyType": "Manual",
            "ManualKey": {
                "ClassId": "storage.LocalKeySetting",
                "ObjectType": "storage.LocalKeySetting",
                "IsExistingKeySet": false,
                "IsNewKeySet": true
            },
            "RemoteKey": {
                "ClassId": "storage.RemoteKeySetting",
                "ObjectType": "storage.RemoteKeySetting",
                "AuthCredentials": {
                    "ClassId": "storage.KmipAuthCredentials",
                    "ObjectType": "storage.KmipAuthCredentials",
                    "IsPasswordSet": false,
                    "UseAuthentication": false,
                    "Username": ""
                },
                "IsExistingKeySet": false,
                "PrimaryServer": {
                    "ClassId": "storage.KmipServer",
                    "ObjectType": "storage.KmipServer",
                    "EnableDriveSecurity": false,
                    "IpAddress": "",
                    "Port": 5696,
                    "Timeout": 60
                },
                "SecondaryServer": {
                    "ClassId": "storage.KmipServer",
                    "ObjectType": "storage.KmipServer",
                    "EnableDriveSecurity": false,
                    "IpAddress": "",
                    "Port": 5696,
                    "Timeout": 60
                },
                "ServerCertificate": ""
            }
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_passphrase(passphrase, field_name):
    """
    Validate that a passphrase meets security requirements.
    """
    if not passphrase:
        return True, None
    if len(passphrase) < 8:
        return False, f"{field_name} must be at least 8 characters long"
    has_upper = any(c.isupper() for c in passphrase)
    has_lower = any(c.islower() for c in passphrase)
    has_digit = any(c.isdigit() for c in passphrase)
    has_special = any(not c.isalnum() for c in passphrase)
    if not (has_upper and has_lower and has_digit and has_special):
        return False, f"{field_name} must include at least one uppercase letter, one lowercase letter, one number, and one special character"
    return True, None


def validate_drive_security_configuration(module):
    """
    Validate Drive Security Policy configuration parameters.
    """
    params = module.params
    if params['state'] == 'present':
        manual_key = params.get('manual_key')
        remote_key = params.get('remote_key')
        if not manual_key and not remote_key:
            module.fail_json(msg="Either manual_key or remote_key must be specified")
        if manual_key and remote_key:
            module.fail_json(msg="Cannot specify both manual_key and remote_key - choose one key management type")
        if manual_key:
            if not manual_key.get('new_key'):
                module.fail_json(msg="manual_key.new_key is required when using manual key management")
            is_valid, error_msg = validate_passphrase(manual_key.get('new_key'), 'manual_key.new_key')
            if not is_valid:
                module.fail_json(msg=error_msg)
            if manual_key.get('existing_key'):
                is_valid, error_msg = validate_passphrase(manual_key.get('existing_key'), 'manual_key.existing_key')
                if not is_valid:
                    module.fail_json(msg=error_msg)
        elif remote_key:
            if not remote_key.get('server_certificate'):
                module.fail_json(msg="remote_key.server_certificate is required when using remote key management (KMIP)")
            primary_server = remote_key.get('primary_server', {})
            secondary_server = remote_key.get('secondary_server', {})
            primary_enabled = primary_server.get('enable_drive_security', False)
            secondary_enabled = secondary_server.get('enable_drive_security', False)
            if not primary_enabled and not secondary_enabled:
                module.fail_json(msg="At least one of primary_server or secondary_server must be enabled")
            if primary_enabled:
                if not primary_server.get('ip_address'):
                    module.fail_json(msg="primary_server.ip_address is required when primary_server is enabled")
                port = primary_server.get('port', 5696)
                if port < 1024 or port > 65535:
                    module.fail_json(msg="primary_server.port must be between 1024 and 65535")
                timeout = primary_server.get('timeout', 60)
                if timeout < 1 or timeout > 250:
                    module.fail_json(msg="primary_server.timeout must be between 1 and 250")
            if secondary_enabled:
                if not secondary_server.get('ip_address'):
                    module.fail_json(msg="secondary_server.ip_address is required when secondary_server is enabled")
                port = secondary_server.get('port', 5696)
                if port < 1024 or port > 65535:
                    module.fail_json(msg="secondary_server.port must be between 1024 and 65535")
                timeout = secondary_server.get('timeout', 60)
                if timeout < 1 or timeout > 250:
                    module.fail_json(msg="secondary_server.timeout must be between 1 and 250")
            if remote_key.get('use_authentication'):
                if not remote_key.get('username'):
                    module.fail_json(msg="remote_key.username is required when use_authentication is true")
            if remote_key.get('existing_key'):
                is_valid, error_msg = validate_passphrase(remote_key.get('existing_key'), 'remote_key.existing_key')
                if not is_valid:
                    module.fail_json(msg=error_msg)


def build_manual_key_config(manual_key):
    """
    Build ManualKey configuration for API body.
    """
    manual_key_config = {
        'NewKey': manual_key['new_key']
    }
    if manual_key.get('existing_key'):
        manual_key_config['ExistingKey'] = manual_key['existing_key']
    else:
        manual_key_config['ExistingKey'] = ''
    return manual_key_config


def build_remote_key_config(remote_key):
    """
    Build RemoteKey configuration for API body.
    """
    remote_key_config = {}
    auth_credentials = {
        'UseAuthentication': remote_key.get('use_authentication', False)
    }
    if remote_key.get('use_authentication'):
        auth_credentials['Username'] = remote_key['username']
        if remote_key.get('password'):
            auth_credentials['Password'] = remote_key['password']
    remote_key_config['AuthCredentials'] = auth_credentials
    remote_key_config['ServerCertificate'] = remote_key['server_certificate']
    primary_server = remote_key.get('primary_server', {})
    primary_server_config = {
        'EnableDriveSecurity': primary_server.get('enable_drive_security', False)
    }
    if primary_server.get('enable_drive_security'):
        primary_server_config['IpAddress'] = primary_server['ip_address']
        primary_server_config['Port'] = primary_server.get('port', 5696)
        primary_server_config['Timeout'] = primary_server.get('timeout', 60)
    else:
        primary_server_config['IpAddress'] = ''
        primary_server_config['Port'] = 5696
        primary_server_config['Timeout'] = 60
    remote_key_config['PrimaryServer'] = primary_server_config
    secondary_server = remote_key.get('secondary_server', {})
    secondary_server_config = {
        'EnableDriveSecurity': secondary_server.get('enable_drive_security', False)
    }
    if secondary_server.get('enable_drive_security'):
        secondary_server_config['IpAddress'] = secondary_server['ip_address']
        secondary_server_config['Port'] = secondary_server.get('port', 5696)
        secondary_server_config['Timeout'] = secondary_server.get('timeout', 60)
    else:
        secondary_server_config['IpAddress'] = ''
        secondary_server_config['Port'] = 5696
        secondary_server_config['Timeout'] = 60
    remote_key_config['SecondaryServer'] = secondary_server_config
    if remote_key.get('existing_key'):
        remote_key_config['ExistingKey'] = remote_key['existing_key']
    else:
        remote_key_config['ExistingKey'] = ''
    return remote_key_config


def build_api_body(intersight):
    """
    Build the API body for Drive Security Policy configuration.
    """
    params = intersight.module.params
    if params['state'] == 'present':
        intersight.api_body = {
            'Organization': {
                'Name': params['organization'],
            },
            'Name': params['name']
        }
        intersight.set_tags_and_description()
        key_setting = {}
        if params.get('manual_key'):
            key_setting['KeyType'] = 'Manual'
            key_setting['ManualKey'] = build_manual_key_config(params['manual_key'])
        elif params.get('remote_key'):
            key_setting['KeyType'] = 'Kmip'
            key_setting['RemoteKey'] = build_remote_key_config(params['remote_key'])
        intersight.api_body['KeySetting'] = key_setting


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        manual_key=dict(
            type='dict',
            no_log=True,
            options=dict(
                new_key=dict(type='str', required=True, no_log=True),
                existing_key=dict(type='str', no_log=True)
            )
        ),
        remote_key=dict(
            type='dict',
            no_log=True,
            options=dict(
                primary_server=dict(
                    type='dict',
                    options=dict(
                        enable_drive_security=dict(type='bool', required=True),
                        ip_address=dict(type='str'),
                        port=dict(type='int', default=5696),
                        timeout=dict(type='int', default=60)
                    )
                ),
                secondary_server=dict(
                    type='dict',
                    options=dict(
                        enable_drive_security=dict(type='bool', required=True),
                        ip_address=dict(type='str'),
                        port=dict(type='int', default=5696),
                        timeout=dict(type='int', default=60)
                    )
                ),
                server_certificate=dict(type='str', required=True),
                use_authentication=dict(type='bool', default=False),
                username=dict(type='str'),
                password=dict(type='str', no_log=True),
                existing_key=dict(type='str', no_log=True)
            )
        )
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    if module.params['state'] == 'present':
        validate_drive_security_configuration(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    build_api_body(intersight)
    resource_path = '/storage/DriveSecurityPolicies'
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2026, Maksym Bakurevych <maxim.bakurevy@gmail.com>
# Copyright (c) 2020, Lee Goolsbee <lgoolsbee@atlassian.com>
# Copyright (c) 2020, Michal Middleton <mm.404@icloud.com>
# Copyright (c) 2017, Steve Pletcher <steve@steve-pletcher.com>
# Copyright (c) 2016, René Moser <mail@renemoser.net>
# Copyright (c) 2015, Stefan Berggren <nsg@nsg.cc>
# Copyright (c) 2014, Ramon de la Fuente <ramon@delafuente.nl>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: slack
short_description: Send Slack notifications
description:
  - The M(community.general.slack) module sends notifications to U(http://slack.com) using the Incoming WebHook integration.
author: "Ramon de la Fuente (@ramondelafuente)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  domain:
    type: str
    description:
      - "When using new format 'Webhook token' and WebAPI tokens: this can be V(slack.com) or V(slack-gov.com) and is ignored
        otherwise."
      - "When using old format 'Webhook token': Slack (sub)domain for your environment without protocol. (For example V(example.slack.com).)
        in Ansible 1.8 and beyond, this is deprecated and may be ignored. See token documentation for information."
  token:
    type: str
    description:
      - Slack integration token. This authenticates you to the Slack service. Make sure to use the correct type of token,
        depending on what method you use.
      - 'Webhook token: Prior to Ansible 1.8, a token looked like V(3Ffe373sfhRE6y42Fg3rvf4GlK). In Ansible 1.8 and above,
        Ansible adapts to the new Slack API where tokens look like V(G922VJP24/D921DW937/3Ffe373sfhRE6y42Fg3rvf4GlK). If tokens
        are in the new format then Slack ignores any value of domain except V(slack.com) or V(slack-gov.com). If the token
        is in the old format the domain is required. Ansible has no control of when Slack is going to remove the old API.
        When Slack does that the old format is going to cease working. B(Please keep in mind the tokens are not the API tokens
        but are the webhook tokens.) In Slack these are found in the webhook URL which are obtained under the apps and integrations.
        The incoming webhooks can be added in that area. In some cases this may be locked by your Slack admin and you must
        request access. It is there that the incoming webhooks can be added. The key is on the end of the URL given to you
        in that section.'
      - "WebAPI token: Slack WebAPI requires a personal, bot or work application token. These tokens start with V(xoxp-),
        V(xoxb-) or V(xoxa-), for example V(xoxb-1234-56789abcdefghijklmnopqrstuvwxyz). WebAPI token is required if you intend to receive
        thread_id. See Slack's documentation (U(https://api.slack.com/docs/token-types)) for more information."
    required: true
  msg:
    type: str
    description:
      - Message to send. Note that the module does not handle escaping characters. Plain-text angle brackets and ampersands
        should be converted to HTML entities (for example C(&) to C(&amp;)) before sending. See Slack's documentation
        (U(https://api.slack.com/docs/message-formatting))
        for more.
  channel:
    type: str
    description:
      - Channel to send the message to. If absent, the message goes to the channel selected for the O(token).
  thread_id:
    description:
      - Optional. Timestamp of parent message to thread this message, see U(https://api.slack.com/docs/message-threading).
    type: str
  message_id:
    description:
      - Optional. Message ID to edit, instead of posting a new message.
      - If supplied O(channel) must be in form of C(C0xxxxxxx). use C({{ slack_response.channel }}) to get RV(ignore:channel)
        from previous task run.
      - The token needs history scope to get information on the message to edit (C(channels:history,groups:history,mpim:history,im:history)).
      - Corresponds to C(ts) in the Slack API (U(https://api.slack.com/messaging/modifying)).
    type: str
    version_added: 1.2.0
  username:
    type: str
    description:
      - This is the sender of the message.
    default: "Ansible"
  icon_url:
    type: str
    description:
      - URL for the message sender's icon.
    default: https://docs.ansible.com/favicon/favicon.ico
  icon_emoji:
    type: str
    description:
      - Emoji for the message sender. See Slack documentation for options.
      - If O(icon_emoji) is set, O(icon_url) is not used.
  link_names:
    type: int
    description:
      - Automatically create links for channels and usernames in O(msg).
    default: 1
    choices:
      - 1
      - 0
  parse:
    type: str
    description:
      - Setting for the message parser at Slack.
    choices:
      - 'full'
      - 'none'
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated. This should only be used on personally controlled sites using self-signed
        certificates.
    type: bool
    default: true
  color:
    type: str
    description:
      - Allow text to use default colors - use the default of V(normal) to not send a custom color bar at the start of the
        message.
      - Allowed values for color can be one of V(normal), V(good), V(warning), V(danger), any valid 3 digit or 6 digit hex
        color value.
    default: 'normal'
  attachments:
    type: list
    elements: dict
    description:
      - Define a list of attachments. This list mirrors the Slack JSON API.
      - For more information, see U(https://api.slack.com/docs/attachments).
  blocks:
    description:
      - Define a list of blocks. This list mirrors the Slack JSON API.
      - For more information, see U(https://api.slack.com/block-kit).
    type: list
    elements: dict
    version_added: 1.0.0
  prepend_hash:
    type: str
    description:
      - Setting for automatically prepending a V(#) symbol on the passed in O(channel).
      - The V(auto) method prepends a V(#) unless O(channel) starts with one of V(#), V(@), V(C0), V(GF), V(G0), V(CP). These
        prefixes only cover a small set of the prefixes that should not have a V(#) prepended. Since an exact condition which
        O(channel) values must not have the V(#) prefix is not known, the value V(auto) for this option is deprecated in the
        future. It is best to explicitly set O(prepend_hash=always) or O(prepend_hash=never) to obtain the needed behavior.
      - Before community.general 12.0.0, the default was V(auto). It has been deprecated since community.general 10.2.0.
      - Note that V(auto) will be deprecated in a future version.
      # TODO: Deprecate 'auto' in community.general 13.0.0
    default: never
    choices:
      - 'always'
      - 'never'
      - 'auto'
    version_added: 6.1.0
  files:
    type: list
    elements: dict
    description:
      - A list of files to be uploaded to Slack.
      - >
        Each list item should be a dictionary containing O(files[].path)
        (absolute or relative path to the file) and optionally
        O(files[].name) (the filename as it will appear in Slack).
      - If O(msg), O(attachments), or O(blocks) are provided, the files are attached as a reply to that message (creating a thread).
      - If no message content is provided, the files are uploaded as a standalone post in the specified O(channel).
      - "Note: File uploading requires a WebAPI token (starting with V(xoxb-) or V(xoxp-))."
      - "It does not work with standard Incoming Webhook URLs (the ones with tokens like V(T.../B.../...) )."
      - The app must have C(files:write) and C(chat:write) scopes in your Slack App settings and must be invited to the channel.
    suboptions:
      path:
        type: path
        required: true
        description:
          - The local path to the file to be uploaded.
      name:
        type: str
        description:
          - The name of the file as it should appear in Slack.
          - If not provided, the base name of the С(path) will be used.
    version_added: 13.1.0
  fail_on_file_error:
    type: bool
    description:
      - If V(true), the module fails if a file is missing or encounters an upload error.
      - If V(false), the module issues a warning and continue processing the next file.
    default: true
    version_added: 13.1.0
"""

EXAMPLES = r"""
- name: Send notification message via Slack
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: '{{ inventory_hostname }} completed'
  delegate_to: localhost

- name: Send notification message via Slack all options
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: '{{ inventory_hostname }} completed'
    channel: '#ansible'
    thread_id: '1539917263.000100'
    username: 'Ansible on {{ inventory_hostname }}'
    icon_url: http://www.example.com/some-image-file.png
    link_names: 0
    parse: 'none'
  delegate_to: localhost

- name: Insert a color bar in front of the message for visibility purposes and use the default webhook icon and name configured
    in Slack
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: '{{ inventory_hostname }} is alive!'
    color: good
    username: ''
    icon_url: ''

- name: Insert a color bar in front of the message with valid hex color value
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: 'This message uses color in hex value'
    color: '#00aacc'
    username: ''
    icon_url: ''

- name: Use the attachments API
  community.general.slack:
    token: thetoken/generatedby/slack
    attachments:
      - text: Display my system load on host A and B
        color: '#ff00dd'
        title: System load
        fields:
          - title: System A
            value: "load average: 0,74, 0,66, 0,63"
            short: true
          - title: System B
            value: 'load average: 5,16, 4,64, 2,43'
            short: true

- name: Use the blocks API
  community.general.slack:
    token: thetoken/generatedby/slack
    blocks:
      - type: section
        text:
          type: mrkdwn
          text: |-
            *System load*
            Display my system load on host A and B
      - type: context
        elements:
          - type: mrkdwn
            text: |-
              *System A*
              load average: 0,74, 0,66, 0,63
          - type: mrkdwn
            text: |-
              *System B*
              load average: 5,16, 4,64, 2,43

- name: Send a message with a link using Slack markup
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: We sent this message using <https://www.ansible.com|Ansible>!

- name: Send a message with angle brackets and ampersands
  community.general.slack:
    token: thetoken/generatedby/slack
    msg: This message has &lt;brackets&gt; &amp; ampersands in plain text.

- name: Initial Threaded Slack message
  community.general.slack:
    channel: '#ansible'
    token: xoxb-1234-56789abcdefghijklmnopqrstuvwxyz
    msg: 'Starting a thread with my initial post.'
  register: slack_response
- name: Add more info to thread
  community.general.slack:
    channel: '#ansible'
    token: xoxb-1234-56789abcdefghijklmnopqrstuvwxyz
    thread_id: "{{ slack_response['ts'] }}"
    color: good
    msg: 'And this is my threaded response!'

- name: Send a message to be edited later on
  community.general.slack:
    token: thetoken/generatedby/slack
    channel: '#ansible'
    msg: Deploying something...
  register: slack_response
- name: Edit message
  community.general.slack:
    token: thetoken/generatedby/slack
    # The 'channel' option does not accept the channel name. It must use the 'channel_id',
    # which can be retrieved for example from 'slack_response' from the previous task.
    channel: "{{ slack_response.channel }}"
    msg: Deployment complete!
    message_id: "{{ slack_response.ts }}"
- name: Send file to Slack
  community.general.slack:
    token: "xoxb-1234-56789abcdefghijklmnopqrstuvwxyz"
    channel: "channel-id"
    fail_on_file_error: false # Optional, defaults to true
    # If you want to sent message to channel without threads,
    # you dont need to use msg parameter
    msg: "Here is the file you asked for"
    files:
      - path: "./first.py" # file in your os
        # File name in Slack. If not provided, it will be the same as path,
        # so in this case "first.py":
        name: "test_report.py"
      - path: "./test_file.txt"
        name: "test_report.txt"
- name: Send file to Slack threads
  community.general.slack:
    token: "xoxb-1234-56789abcdefghijklmnopqrstuvwxyz"
    channel: "channel-id"
    thread_id: "thread-id" # if you want to send file to a specific thread
    files:
      - path: "./first.py" # file in your os
        # File name in Slack. If not provided, it will be the same as path,
        # so in this case "first.py":
        name: "test_report.py"
"""
import os
import re
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

# Escaping quotes and apostrophes to avoid ending string prematurely in ansible call.
# We do not escape other characters used as Slack metacharacters (e.g. &, <, >).
escape_table = {
    '"': '"',
    "'": "'",
}


def is_valid_hex_color(color_choice):
    return bool(re.match(r"^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$", color_choice))


def escape_quotes(text):
    """Backslash any quotes within text."""
    return "".join(escape_table.get(c, c) for c in text)


def recursive_escape_quotes(obj, keys):
    """Recursively escape quotes inside supplied keys inside block kit objects"""
    if isinstance(obj, dict):
        escaped = {}
        for k, v in obj.items():
            if isinstance(v, str) and k in keys:
                escaped[k] = escape_quotes(v)
            else:
                escaped[k] = recursive_escape_quotes(v, keys)
    elif isinstance(obj, list):
        escaped = [recursive_escape_quotes(v, keys) for v in obj]
    else:
        escaped = obj
    return escaped


def build_payload_for_slack(
    text,
    channel,
    thread_id,
    username,
    icon_url,
    icon_emoji,
    link_names,
    parse,
    color,
    attachments,
    blocks,
    message_id,
    prepend_hash,
):
    payload = {}
    if color == "normal" and text is not None:
        payload = dict(text=escape_quotes(text))
    elif text is not None:
        # With a custom color we have to set the message as attachment, and explicitly turn markdown parsing on for it.
        payload = dict(attachments=[dict(text=escape_quotes(text), color=color, mrkdwn_in=["text"])])
    if channel is not None:
        if prepend_hash == "auto":
            if channel.startswith(("#", "@", "C0", "GF", "G0", "CP")):
                payload["channel"] = channel
            else:
                payload["channel"] = f"#{channel}"
        elif prepend_hash == "always":
            payload["channel"] = f"#{channel}"
        elif prepend_hash == "never":
            payload["channel"] = channel
    if thread_id is not None:
        payload["thread_ts"] = thread_id
    if username is not None:
        payload["username"] = username
    if icon_emoji is not None:
        payload["icon_emoji"] = icon_emoji
    else:
        payload["icon_url"] = icon_url
    if link_names is not None:
        payload["link_names"] = link_names
    if parse is not None:
        payload["parse"] = parse
    if message_id is not None:
        payload["ts"] = message_id

    if attachments is not None:
        if "attachments" not in payload:
            payload["attachments"] = []

    if attachments is not None:
        attachment_keys_to_escape = [
            "title",
            "text",
            "author_name",
            "pretext",
            "fallback",
        ]
        for attachment in attachments:
            for key in attachment_keys_to_escape:
                if key in attachment:
                    attachment[key] = escape_quotes(attachment[key])

            if "fallback" not in attachment:
                attachment["fallback"] = attachment["text"]

            payload["attachments"].append(attachment)

    if blocks is not None:
        block_keys_to_escape = ["text", "alt_text"]
        payload["blocks"] = recursive_escape_quotes(blocks, block_keys_to_escape)

    return payload


def validate_slack_domain(domain):
    return domain if domain in ("slack.com", "slack-gov.com") else "slack.com"


def get_slack_message(module, domain, token, channel, ts):
    headers = {
        "Content-Type": "application/json; charset=UTF-8",
        "Accept": "application/json",
        "Authorization": f"Bearer {token}",
    }
    qs = urlencode(
        {
            "channel": channel,
            "ts": ts,
            "limit": 1,
            "inclusive": "true",
        }
    )
    domain = validate_slack_domain(domain)
    url = f"https://{domain}/api/conversations.history?{qs}"
    response, info = fetch_url(module=module, url=url, headers=headers, method="GET")
    if info["status"] != 200:
        module.fail_json(msg="failed to get slack message")
    data = module.from_json(response.read())
    if data.get("ok") is False:
        module.fail_json(msg=f"failed to get slack message: {data}")
    if len(data["messages"]) < 1:
        module.fail_json(msg=f"no messages matching ts: {ts}")
    if len(data["messages"]) > 1:
        module.fail_json(msg=f"more than 1 message matching ts: {ts}")
    return data["messages"][0]


def do_notify_slack(module, domain, token, payload):
    use_webapi = False
    if token.count("/") >= 2:
        # New style webhook token
        domain = validate_slack_domain(domain)
        slack_uri = f"https://hooks.{domain}/services/{token}"
    elif re.match(r"^xox[abp]-\S+$", token):
        domain = validate_slack_domain(domain)
        slack_uri = f"https://{domain}/api/{'chat.update' if 'ts' in payload else 'chat.postMessage'}"
        use_webapi = True
    else:
        if not domain:
            module.fail_json(
                msg="Slack has updated its webhook API. You need to specify a token of the form "
                "XXXX/YYYY/ZZZZ in your playbook"
            )
        slack_uri = f"https://{domain}/services/hooks/incoming-webhook?token={token}"

    headers = {
        "Content-Type": "application/json; charset=UTF-8",
        "Accept": "application/json",
    }
    if use_webapi:
        headers["Authorization"] = f"Bearer {token}"

    data = module.jsonify(payload)
    response, info = fetch_url(module=module, url=slack_uri, headers=headers, method="POST", data=data)

    if info["status"] != 200:
        if use_webapi:
            obscured_incoming_webhook = slack_uri
        else:
            obscured_incoming_webhook = f"https://hooks.{domain}/services/[obscured]"
        module.fail_json(msg=f" failed to send {data} to {obscured_incoming_webhook}: {info['msg']}")

    # each API requires different handling
    if use_webapi:
        return module.from_json(response.read())
    else:
        return {"webhook": "ok"}


def upload_slack_files(module, token, channel, files, thread_ts=None, fail_on_file_error=True):
    if not files:
        return {"ok": False, "msg": "No files provided"}

    uploaded_ids = []
    headers = {"Authorization": f"Bearer {token}"}

    for f_item in files:
        f_path = f_item["path"]
        f_name = f_item["name"] or os.path.basename(f_path)

        if not os.path.exists(f_path):
            error_msg = f"File {f_path} not found."
            if fail_on_file_error:
                module.fail_json(msg=error_msg)
            else:
                module.warn(f"{error_msg} Skipping.")
                continue

        file_size = os.path.getsize(f_path)
        url_get = f"https://slack.com/api/files.getUploadURLExternal?filename={f_name}&length={file_size}"

        resp, info = fetch_url(module, url_get, headers=headers, method="GET")

        if info["status"] != 200:
            module.fail_json(
                msg=f"Failed to get upload URL for {f_name}. Slack API endpoint returned HTTP {info['status']}.",
                details=info.get("msg", "No HTTP error message provided"),
            )

        res = module.from_json(resp.read())

        if not res.get("ok"):
            error_code = res.get("error", "unknown_error")
            fatal_errors = ["invalid_auth", "unknown_method", "missing_scope", "account_inactive"]
            if error_code in fatal_errors:
                module.fail_json(
                    msg=f"Fatal Slack API error occurred for {f_name}. Operation aborted.", error=error_code
                )
            module.warn(f"Slack API error for {f_name}: {error_code}")
            continue

        try:
            with open(f_path, "rb") as f:
                file_data = f.read()

            u_resp, u_info = fetch_url(
                module,
                res["upload_url"],
                data=file_data,
                method="POST",
                headers={"Content-Type": "application/octet-stream"},
            )

            if u_info["status"] != 200:
                module.warn(f"Failed to upload bits for {f_name}. Status: {u_info['status']}")
                continue

        except Exception as e:
            module.warn(f"Failed to upload bits for {f_name}: {e}")
            continue

        uploaded_ids.append({"id": res["file_id"], "title": f_name})

    if uploaded_ids:
        completion_payload = {"files": uploaded_ids, "channel_id": channel, "initial_comment": "Attached Files:"}

        if thread_ts:
            completion_payload["thread_ts"] = thread_ts

        f_url = "https://slack.com/api/files.completeUploadExternal"
        final_headers = headers.copy()
        final_headers["Content-Type"] = "application/json; charset=utf-8"

        resp, info = fetch_url(
            module, f_url, headers=final_headers, method="POST", data=module.jsonify(completion_payload)
        )

        if info["status"] != 200:
            return {"ok": False, "msg": f"Failed to complete upload. Status: {info['status']}"}

        return module.from_json(resp.read())

    return {"ok": False, "msg": "No files were successfully uploaded"}


def main():
    module = AnsibleModule(
        argument_spec=dict(
            domain=dict(type="str"),
            token=dict(type="str", required=True, no_log=True),
            msg=dict(type="str"),
            channel=dict(type="str"),
            thread_id=dict(type="str"),
            username=dict(type="str", default="Ansible"),
            icon_url=dict(type="str", default="https://docs.ansible.com/favicon/favicon.ico"),
            icon_emoji=dict(type="str"),
            link_names=dict(type="int", default=1, choices=[0, 1]),
            parse=dict(type="str", choices=["none", "full"]),
            validate_certs=dict(default=True, type="bool"),
            color=dict(type="str", default="normal"),
            attachments=dict(type="list", elements="dict"),
            blocks=dict(type="list", elements="dict"),
            message_id=dict(type="str"),
            prepend_hash=dict(type="str", choices=["always", "never", "auto"], default="never"),
            fail_on_file_error=dict(type="bool", default=True),
            files=dict(
                type="list",
                elements="dict",
                options=dict(
                    path=dict(type="path", required=True),
                    name=dict(type="str"),
                ),
            ),
        ),
        supports_check_mode=True,
    )

    domain = module.params["domain"]
    token = module.params["token"]
    text = module.params["msg"]
    channel = module.params["channel"]
    thread_id = module.params["thread_id"]
    username = module.params["username"]
    icon_url = module.params["icon_url"]
    icon_emoji = module.params["icon_emoji"]
    link_names = module.params["link_names"]
    parse = module.params["parse"]
    color = module.params["color"]
    attachments = module.params["attachments"]
    blocks = module.params["blocks"]
    message_id = module.params["message_id"]
    prepend_hash = module.params["prepend_hash"]
    fail_on_file_error = module.params["fail_on_file_error"]
    files = module.params["files"]
    is_webhook = re.match(r"^T[A-Z0-9]+/B[A-Z0-9]+/[A-Za-z0-9]+$", token)
    is_api_token = re.match(r"^xox[bpa]-", token)
    if not (is_webhook or is_api_token):
        module.fail_json(
            msg="The token provided is not a valid Slack token. "
            "Webhooks should look like T.../B.../X... and "
            "API tokens should start with xoxb-, xoxp-, or xoxa-."
        )
    color_choices = ["normal", "good", "warning", "danger"]
    if color not in color_choices and not is_valid_hex_color(color):
        module.fail_json(
            msg=f"Color value specified should be either one of {color_choices} or any valid hex value with length 3 or 6."
        )

    changed = True

    # if updating an existing message, we can check if there's anything to update
    if message_id is not None:
        changed = False
        msg = get_slack_message(module, domain, token, channel, message_id)
        for key in ("icon_url", "icon_emoji", "link_names", "color", "attachments", "blocks"):
            if msg.get(key) != module.params.get(key):
                changed = True
                break
        # if check mode is active, we shouldn't do anything regardless.
        # if changed=False, we don't need to do anything, so don't do it.
        if module.check_mode or not changed:
            module.exit_json(changed=changed, ts=msg["ts"], channel=msg["channel"])
    elif module.check_mode:
        module.exit_json(changed=changed)

    payload = build_payload_for_slack(
        text,
        channel,
        thread_id,
        username,
        icon_url,
        icon_emoji,
        link_names,
        parse,
        color,
        attachments,
        blocks,
        message_id,
        prepend_hash,
    )

    has_message_content = bool(text or attachments or blocks)
    slack_response = {}
    is_success = False

    if has_message_content:
        slack_response = do_notify_slack(module, domain, token, payload)
        # Check success for both WebAPI (ok: true) and incoming webhooks
        # (webhook: ok)
        is_success = slack_response.get("ok") or slack_response.get("webhook") == "ok"
    else:
        is_success = True

    file_upload_res = None
    if files and is_success:
        target_channel = slack_response.get("channel") or channel
        target_thread = slack_response.get("ts") or thread_id

        file_upload_res = upload_slack_files(
            module, token, target_channel, files, thread_ts=target_thread, fail_on_file_error=fail_on_file_error
        )

        # If sending only files, overall success depends on the upload result
        if not has_message_content:
            is_success = file_upload_res.get("ok", False)

    if is_success:
        # Exit with plain OK from WebHook, since we don't have more information
        # If we get 200 from webhook, the only answer is OK
        if "ok" not in slack_response and slack_response.get("webhook") == "ok" and not files:
            module.exit_json(msg="OK", changed=True)

        result = {
            "changed": True,
            "api": slack_response if has_message_content else {"status": "files_only_upload"},
            "payload": module.jsonify(payload) if has_message_content else None,
        }

        if file_upload_res:
            result["files_upload"] = file_upload_res

        if "ts" in slack_response:
            result.update({"ts": slack_response["ts"], "channel": slack_response["channel"]})
        elif file_upload_res and "files" in file_upload_res:
            result.update({"channel": channel})

        module.exit_json(**result)
    else:
        error_msg = slack_response.get("error") or (file_upload_res.get("msg") if file_upload_res else "Unknown error")
        module.fail_json(msg="Slack operation failed", error=error_msg)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, INSPQ (@elfelip)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_user
short_description: Create and configure a user in Keycloak
description:
  - This module creates, removes, or updates Keycloak users.
version_added: 7.1.0
options:
  auth_username:
    aliases: []
  realm:
    description:
      - The name of the realm in which is the client.
    default: master
    type: str
  username:
    description:
      - Username for the user.
    required: true
    type: str
  id:
    description:
      - ID of the user on the Keycloak server if known.
    type: str
  enabled:
    description:
      - Enabled user.
    type: bool
  email_verified:
    description:
      - Set or reset the C(emailVerified) flag of the user.
      - When O(email_verified_behavior=no_defaults), the default value of this option becomes C(null) and
        that causes the module not to change any existing value for that attribute.
    type: bool
    aliases:
      - emailVerified
  first_name:
    description:
      - The user's first name.
    type: str
    aliases:
      - firstName
  last_name:
    description:
      - The user's last name.
    type: str
    aliases:
      - lastName
  email:
    description:
      - User email.
    type: str
  federation_link:
    description:
      - Federation Link.
    type: str
    aliases:
      - federationLink
  service_account_client_id:
    description:
      - Description of the client Application.
    type: str
    aliases:
      - serviceAccountClientId
  client_consents:
    description:
      - Client Authenticator Type.
    type: list
    elements: dict
    default: []
    aliases:
      - clientConsents
    suboptions:
      client_id:
        description:
          - Client ID of the client role. Not the technical ID of the client.
        type: str
        required: true
        aliases:
          - clientId
      roles:
        description:
          - List of client roles to assign to the user.
        type: list
        required: true
        elements: str
  groups:
    description:
      - List of groups for the user.
      - Groups can be referenced by their name, like V(staff), or their path, like V(/staff/engineering). The path syntax
        allows you to reference subgroups, which is not possible otherwise.
      - Using the path is possible since community.general 10.6.0.
    type: list
    elements: dict
    default: []
    suboptions:
      name:
        description:
          - Name of the group.
        type: str
      state:
        description:
          - Control whether the user must be member of this group or not.
        choices: ["present", "absent"]
        default: present
        type: str
  credentials:
    description:
      - User credentials.
    default: []
    type: list
    elements: dict
    suboptions:
      type:
        description:
          - Credential type.
        type: str
        required: true
      value:
        description:
          - Value of the credential.
        type: str
        required: true
      temporary:
        description:
          - If V(true), the users are required to reset their credentials at next login.
        type: bool
        default: false
  required_actions:
    description:
      - Set or reset a user's required actions.
    type: list
    elements: str
    aliases:
      - requiredActions
  federated_identities:
    description:
      - List of IDPs of user.
    default: []
    type: list
    elements: str
    aliases:
      - federatedIdentities
  attributes:
    description:
      - List of user attributes.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name of the attribute.
        type: str
      values:
        description:
          - Values for the attribute as list.
        type: list
        elements: str
      state:
        description:
          - Control whether the attribute must exists or not.
        choices: ["present", "absent"]
        default: present
        type: str
  access:
    description:
      - List user access.
    type: dict
  disableable_credential_types:
    description:
      - List user Credential Type.
    default: []
    type: list
    elements: str
    aliases:
      - disableableCredentialTypes
  origin:
    description:
      - User origin.
    type: str
  self:
    description:
      - User self administration.
    type: str
  state:
    description:
      - Control whether the user should exists or not.
    choices: ["present", "absent"]
    default: present
    type: str
  force:
    description:
      - If V(true), allows to remove user and recreate it.
    type: bool
    default: false
  email_verified_behavior:
    description:
      - The O(email_verified) option used to have a default value. This caused problems when the
        user expects different behavior from keycloak by default.
      - The default value of this option is V(compatibility), which will ensure that the old default value
        for O(email_verified) is used.
      - When set to V(no_defaults), the module will not change existing values of O(email_verified) if no value is specified.
    type: str
    choices:
      - compatibility
      - no_defaults
    default: compatibility
    version_added: "13.1.0"
extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0
notes:
  - The module does not modify the user ID of an existing user.
author:
  - Philippe Gauthier (@elfelip)
"""

EXAMPLES = r"""
- name: Create a user user1
  community.general.keycloak_user:
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: admin
    auth_password: password
    realm: master
    username: user1
    firstName: user1
    lastName: user1
    email: user1
    enabled: true
    emailVerified: false
    credentials:
      - type: password
        value: password
        temporary: false
    attributes:
      - name: attr1
        values:
          - value1
        state: present
      - name: attr2
        values:
          - value2
        state: absent
    groups:
      - name: group1
        state: present
    state: present

- name: Re-create a User
  community.general.keycloak_user:
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: admin
    auth_password: password
    realm: master
    username: user1
    firstName: user1
    lastName: user1
    email: user1
    enabled: true
    emailVerified: false
    credentials:
      - type: password
        value: password
        temporary: false
    attributes:
      - name: attr1
        values:
          - value1
        state: present
      - name: attr2
        values:
          - value2
        state: absent
    groups:
      - name: group1
        state: present
    state: present

- name: Re-create a User
  community.general.keycloak_user:
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: admin
    auth_password: password
    realm: master
    username: user1
    firstName: user1
    lastName: user1
    email: user1
    enabled: true
    emailVerified: false
    credentials:
      - type: password
        value: password
        temporary: false
    attributes:
      - name: attr1
        values:
          - value1
        state: present
      - name: attr2
        values:
          - value2
        state: absent
    groups:
      - name: group1
        state: present
    state: present
    force: true

- name: Remove User
  community.general.keycloak_user:
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: admin
    auth_password: password
    realm: master
    username: user1
    state: absent
"""

RETURN = r"""
proposed:
  description: Representation of the proposed user.
  returned: on success
  type: dict
existing:
  description: Representation of the existing user.
  returned: on success
  type: dict
end_state:
  description: Representation of the user after module execution.
  returned: on success
  type: dict
user_created:
  description: Indicates whether a user was created.
  returned: in success
  type: bool
  version_added: 12.0.0
"""

import copy

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    camel,
    get_token,
    is_struct_included,
    keycloak_argument_spec,
)


def main():
    argument_spec = keycloak_argument_spec()
    argument_spec["auth_username"]["aliases"] = []
    credential_spec = dict(
        type=dict(type="str", required=True),
        value=dict(type="str", required=True, no_log=True),
        temporary=dict(type="bool", default=False),
    )
    client_consents_spec = dict(
        client_id=dict(type="str", required=True, aliases=["clientId"]),
        roles=dict(type="list", elements="str", required=True),
    )
    attributes_spec = dict(
        name=dict(type="str"),
        values=dict(type="list", elements="str"),
        state=dict(type="str", choices=["present", "absent"], default="present"),
    )
    groups_spec = dict(name=dict(type="str"), state=dict(type="str", choices=["present", "absent"], default="present"))
    meta_args = dict(
        realm=dict(type="str", default="master"),
        self=dict(type="str"),
        id=dict(type="str"),
        username=dict(type="str", required=True),
        first_name=dict(type="str", aliases=["firstName"]),
        last_name=dict(type="str", aliases=["lastName"]),
        email=dict(type="str"),
        enabled=dict(type="bool"),
        email_verified=dict(type="bool", aliases=["emailVerified"]),
        federation_link=dict(type="str", aliases=["federationLink"]),
        service_account_client_id=dict(type="str", aliases=["serviceAccountClientId"]),
        attributes=dict(type="list", elements="dict", options=attributes_spec),
        access=dict(type="dict"),
        groups=dict(type="list", default=[], elements="dict", options=groups_spec),
        disableable_credential_types=dict(
            type="list", default=[], aliases=["disableableCredentialTypes"], elements="str"
        ),
        required_actions=dict(type="list", aliases=["requiredActions"], elements="str"),
        credentials=dict(type="list", default=[], elements="dict", options=credential_spec),
        federated_identities=dict(type="list", default=[], aliases=["federatedIdentities"], elements="str"),
        client_consents=dict(
            type="list", default=[], aliases=["clientConsents"], elements="dict", options=client_consents_spec
        ),
        origin=dict(type="str"),
        state=dict(choices=["absent", "present"], default="present"),
        force=dict(type="bool", default=False),
        email_verified_behavior=dict(type="str", choices=["compatibility", "no_defaults"], default="compatibility"),
    )
    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", diff={}, proposed={}, existing={}, end_state={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    realm = module.params.get("realm")
    state = module.params.get("state")
    force = module.params.get("force")
    username = module.params.get("username")
    groups = module.params.get("groups")

    # If there is no value for email_verified, check if we should to set the old default
    if module.params["email_verified"] is None and module.params["email_verified_behavior"] == "compatibility":
        module.params["email_verified"] = False

    ignored_arguments = list(keycloak_argument_spec().keys()) + [
        "state",
        "realm",
        "force",
        "groups",
        "email_verified_behavior",
    ]

    # Filter and map the parameters names that apply to the user
    user_params = [x for x in module.params if x not in ignored_arguments and module.params[x] is not None]

    before_user = kc.get_user_by_username(username=username, realm=realm)

    if before_user is None:
        before_user = {}

    changeset = {}

    for param in user_params:
        new_param_value = module.params.get(param)
        if param == "attributes" and param in before_user:
            old_value = kc.convert_keycloak_user_attributes_dict_to_module_list(attributes=before_user["attributes"])
        else:
            old_value = before_user[param] if param in before_user else None
        if new_param_value != old_value:
            if old_value is not None and param == "attributes":
                for old_attribute in old_value:
                    old_attribute_found = False
                    for new_attribute in new_param_value:
                        if new_attribute["name"] == old_attribute["name"]:
                            old_attribute_found = True
                    if not old_attribute_found:
                        new_param_value.append(copy.deepcopy(old_attribute))
            if isinstance(new_param_value, dict):
                changeset[camel(param)] = copy.deepcopy(new_param_value)
            else:
                changeset[camel(param)] = new_param_value
    # Prepare the desired values using the existing values (non-existence results in a dict that is save to use as a basis)
    desired_user = copy.deepcopy(before_user)
    desired_user.update(changeset)

    if before_user:
        before_groups = kc.get_user_groups(user_id=before_user["id"], realm=realm)
        before_user["groups"] = before_groups
    else:
        before_groups = []

    result["proposed"] = changeset
    result["existing"] = before_user
    # Default values for user_created
    result["user_created"] = False
    changed = False
    after_user = {}

    if state == "absent":
        if not before_user:
            # Do nothing and exit
            result["msg"] = "User does not exist, doing nothing."
        else:
            # Delete user
            if not module.check_mode:
                kc.delete_user(user_id=before_user["id"], realm=realm)
            result["msg"] = f"User {before_user['username']} deleted"
            changed = True
    else:
        if (not before_user or force) and username is None:
            module.fail_json(msg="username must be specified when creating a new user")

        if force and before_user and not module.check_mode:  # If the force option is set to true
            # Delete the existing user
            kc.delete_user(user_id=before_user["id"], realm=realm)

        if not before_user or force:
            # Create a new user
            if not module.check_mode:
                # Create the user
                after_user = kc.create_user(userrep=desired_user, realm=realm)
                # Add user ID to desired_user for group updates
                desired_user["id"] = after_user["id"]
            else:
                after_user = desired_user

            result["msg"] = f"User {desired_user['username']} created"
            result["user_created"] = True
            changed = True
        else:
            # Update an existing user
            excludes = [
                "access",
                "notBefore",
                "createdTimestamp",
                "totp",
                "credentials",
                "disableableCredentialTypes",
                "groups",
                "clientConsents",
                "federatedIdentities",
            ]
            # Compare users
            if not (
                is_struct_included(desired_user, before_user, excludes, empty_list_result=False)
            ):  # If the new user introduces a change to the existing user
                # Update the user
                if not module.check_mode:
                    after_user = kc.update_user(userrep=desired_user, realm=realm)
                changed = True

            if not after_user:
                # no change
                after_user = desired_user

        # set user groups
        if not module.check_mode:
            changed |= kc.update_user_groups_membership(userrep=desired_user, groups=groups, realm=realm)

        present_groups = [g["name"] for g in groups if g["state"] == "present"]
        absent_groups = [g["name"] for g in groups if g["state"] == "absent"]

        desired_user["groups"] = (set(before_groups) | set(present_groups)) - set(absent_groups)

        if module.check_mode:
            # check if group meberships would have changed
            changed |= not is_struct_included(
                desired_user["groups"], before_user["groups"], excludes, empty_list_result=False
            )
        else:
            after_user["groups"] = kc.get_user_groups(user_id=desired_user["id"], realm=realm)

    if not result["msg"]:
        if changed:
            result["msg"] = f"User {desired_user['username']} updated"
        else:
            result["msg"] = f"No changes made for user {desired_user['username']}"
    result["end_state"] = after_user
    result["changed"] = changed
    result["diff"] = dict(before=before_user, after=after_user)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: administered_licensing_subscription_subscriptions_info
short_description: Information module for administered _licensing _subscription _subscriptions
description:
  - Get all administered _licensing _subscription _subscriptions.
  - List available subscriptions.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  subscriptionIds:
    description:
      - SubscriptionIds query parameter. List of subscription ids to fetch.
    elements: str
    type: list
  organizationIds:
    description:
      - OrganizationIds query parameter. Organizations to get associated subscriptions
        for.
    elements: str
    type: list
  statuses:
    description:
      - Statuses query parameter. List of statuses that returned subscriptions can
        have.
    elements: str
    type: list
  productTypes:
    description:
      - ProductTypes query parameter. List of product types that returned subscriptions
        need to have entitlements for.
    elements: str
    type: list
  skus:
    description:
      - Skus query parameter. List of SKUs that returned subscriptions need to have
        entitlements for.
    elements: str
    type: list
  name:
    description:
      - Name query parameter. Search for subscription name.
    type: str
  startDate:
    description:
      - >
        StartDate query parameter. Filter subscriptions by start date, ISO 8601 format.
        To filter with a range of dates, use 'startDate<option>=?' in the request.
        Accepted options include lt, gt, lte, gte.
    type: str
  endDate:
    description:
      - >
        EndDate query parameter. Filter subscriptions by end date, ISO 8601 format.
        To filter with a range of dates, use 'endDate<option>=?' in the request. Accepted
        options include lt, gt, lte, gte.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for licensing getAdministeredLicensingSubscriptionSubscriptions
    description: Complete reference of the getAdministeredLicensingSubscriptionSubscriptions
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-administered-licensing-subscription-subscriptions
notes:
  - SDK Method used are
    licensing.Licensing.get_administered_licensing_subscription_subscriptions,
  - Paths used are
    get /administered/licensing/subscription/subscriptions,
"""

EXAMPLES = r"""
- name: Get all administered _licensing _subscription _subscriptions
  cisco.meraki.administered_licensing_subscription_subscriptions_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    subscriptionIds: []
    organizationIds: []
    statuses: []
    productTypes: []
    skus: []
    name: string
    startDate: string
    endDate: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "subscriptionId": "string",
        "name": "string",
        "description": "string",
        "status": "string",
        "startDate": "string",
        "endDate": "string",
        "lastUpdatedAt": "string",
        "webOrderId": "string",
        "type": "string",
        "smartAccount": {
          "status": "string",
          "account": {
            "id": "string",
            "name": "string",
            "domain": "string"
          }
        },
        "renewalRequested": true,
        "productTypes": [
          "string"
        ],
        "entitlements": [
          {
            "sku": "string",
            "seats": {
              "assigned": 0,
              "available": 0,
              "limit": 0
            }
          }
        ],
        "counts": {
          "seats": {
            "assigned": 0,
            "available": 0,
            "limit": 0
          },
          "networks": 0,
          "organizations": 0
        },
        "enterpriseAgreement": {
          "suites": [
            "string"
          ]
        }
      }
    ]
"""



#!/usr/bin/python

# Copyright (c) 2023, Andrew Hyatt <andy@hyatt.xyz>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: dnf_config_manager
short_description: Enable or disable dnf repositories using config-manager
version_added: 8.2.0
description:
  - This module enables or disables repositories using the C(dnf config-manager) sub-command.
author: Andrew Hyatt (@ahyattdev) <andy@hyatt.xyz>
requirements:
  - dnf
  - dnf-plugins-core
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Repository ID, for example V(crb).
    default: []
    type: list
    elements: str
  state:
    description:
      - Whether the repositories should be V(enabled) or V(disabled).
    default: enabled
    type: str
    choices: [enabled, disabled]
notes:
  - Does not work with C(dnf5).
seealso:
  - module: ansible.builtin.dnf
  - module: ansible.builtin.yum_repository
"""

EXAMPLES = r"""
- name: Ensure the crb repository is enabled
  community.general.dnf_config_manager:
    name: crb
    state: enabled

- name: Ensure the appstream and zfs repositories are disabled
  community.general.dnf_config_manager:
    name:
      - appstream
      - zfs
    state: disabled
"""

RETURN = r"""
repo_states_pre:
  description: Repo IDs before action taken.
  returned: success
  type: dict
  contains:
    enabled:
      description: Enabled repository IDs.
      returned: success
      type: list
      elements: str
    disabled:
      description: Disabled repository IDs.
      returned: success
      type: list
      elements: str
  sample:
    enabled:
      - appstream
      - baseos
      - crb
    disabled:
      - appstream-debuginfo
      - appstream-source
      - baseos-debuginfo
      - baseos-source
      - crb-debug
      - crb-source
repo_states_post:
  description: Repository states after action taken.
  returned: success
  type: dict
  contains:
    enabled:
      description: Enabled repository IDs.
      returned: success
      type: list
      elements: str
    disabled:
      description: Disabled repository IDs.
      returned: success
      type: list
      elements: str
  sample:
    enabled:
      - appstream
      - baseos
      - crb
    disabled:
      - appstream-debuginfo
      - appstream-source
      - baseos-debuginfo
      - baseos-source
      - crb-debug
      - crb-source
changed_repos:
  description: Repositories changed.
  returned: success
  type: list
  elements: str
  sample: ["crb"]
"""

import re
import typing as t

from ansible.module_utils.basic import AnsibleModule

REPO_ID_RE = re.compile(r"^Repo[-\s]id\s*:\s*(\S+)$", re.IGNORECASE)
REPO_STATUS_RE = re.compile(r"^(?:Repo-)?status\s*:\s*(disabled|enabled)$", re.IGNORECASE)


def get_dnf_version(module, dnf) -> t.Literal[4, 5]:
    rc, out, err = module.run_command([dnf, "--version"], check_rc=True)
    line, separator, rest = out.partition("\n")

    return 5 if re.match(r"^dnf5\s*", line) else 4


def get_repo_states(module, dnf, dnf_v):
    command = [dnf]
    if dnf_v == 4:
        command.extend(["repolist", "--all", "--verbose"])
    else:
        command.extend(["repo", "info", "--all"])

    rc, out, err = module.run_command(command, check_rc=True)

    repos = dict()
    last_repo = ""
    for line in out.split("\n"):
        m = REPO_ID_RE.match(line)
        if m:
            if len(last_repo) > 0:
                module.fail_json(msg="dnf repolist parse failure: parsed another repo id before next status")
            last_repo = m.group(1)
            continue
        m = REPO_STATUS_RE.match(line)
        if m:
            if len(last_repo) == 0:
                module.fail_json(msg="dnf repolist parse failure: parsed status before repo id")
            repos[last_repo] = m.group(1)
            last_repo = ""
    return repos


def set_repo_states(module, dnf, dnf_v, repo_ids, state):
    if dnf_v == 4:
        module.run_command([dnf, "config-manager", "--assumeyes", f"--set-{state}"] + repo_ids, check_rc=True)
    else:
        state = "1" if state == "enabled" else "0"
        opts = map(lambda v: f"{v}.enabled={state}", repo_ids)
        module.run_command([dnf, "config-manager", "setopt"] + list(opts), check_rc=True)


def pack_repo_states_for_return(states):
    enabled = []
    disabled = []
    for repo_id in states:
        if states[repo_id] == "enabled":
            enabled.append(repo_id)
        else:
            disabled.append(repo_id)

    # Sort for consistent results
    enabled.sort()
    disabled.sort()

    return {"enabled": enabled, "disabled": disabled}


def main():
    module_args = dict(
        name=dict(type="list", elements="str", default=[]),
        state=dict(type="str", choices=["enabled", "disabled"], default="enabled"),
    )

    result = dict(changed=False)

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)
    module.run_command_environ_update = dict(LANGUAGE="C", LC_ALL="C")

    dnf = module.get_bin_path("dnf", True)

    dnf_v = get_dnf_version(module, dnf)

    repo_states = get_repo_states(module, dnf, dnf_v)
    result["repo_states_pre"] = pack_repo_states_for_return(repo_states)

    desired_repo_state = module.params["state"]
    names = module.params["name"]

    to_change = []
    for repo_id in names:
        if repo_id not in repo_states:
            module.fail_json(msg=f"did not find repo with ID '{repo_id}' in dnf repolist --all --verbose")
        if repo_states[repo_id] != desired_repo_state:
            to_change.append(repo_id)
    result["changed"] = len(to_change) > 0
    result["changed_repos"] = to_change

    if module.check_mode:
        module.exit_json(**result)

    if len(to_change) > 0:
        set_repo_states(module, dnf, dnf_v, to_change, desired_repo_state)

    repo_states_post = get_repo_states(module, dnf, dnf_v)
    result["repo_states_post"] = pack_repo_states_for_return(repo_states_post)

    for repo_id in to_change:
        if repo_states_post[repo_id] != desired_repo_state:
            module.fail_json(msg=f"dnf config-manager failed to make '{repo_id}' {desired_repo_state}")

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_appliance_performance_info
short_description: Information module for devices _appliance _performance
description:
  - Get all devices _appliance _performance. - > Return the performance score for
    a single MX. Only primary MX devices supported. If no data is available, a 204
    error code is returned.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
      - Serial path parameter.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 30 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 14 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be greater than or equal to 30 minutes and be less than
        or equal to 14 days. The default is 30 minutes.
    type: float
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance getDeviceAppliancePerformance
    description: Complete reference of the getDeviceAppliancePerformance API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-appliance-performance
notes:
  - SDK Method used are
    appliance.Appliance.get_device_appliance_performance,
  - Paths used are
    get /devices/{serial}/appliance/performance,
"""

EXAMPLES = r"""
- name: Get all devices _appliance _performance
  cisco.meraki.devices_appliance_performance_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    t0: string
    t1: string
    timespan: 0
    serial: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "perfScore": 0
    }
"""
